/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
*******************************************************************************/
/* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.   */
/*******************************************************************************
* File Name : const_def.h
* Version : 2.0
* Device(s) : RL78/G14
* Tool-Chain : RL78 IAR Embedded Workbench V1.20.x
* OS : None
* H/W Platform : YRMCKITRL78G14 Rev 2.0
* Description  : internal constants definitions
* Operation :
* Limitations : None
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0    Production release plus rebuild for new IAR EW release
*                              modified file organisation to support object creation
******************************************************************************/


#ifndef _CONST_DEF_H
#define _CONST_DEF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "customize.h"
#include "mcrplib.h"

/******************************************************************************
Macro definitions
******************************************************************************/

/*** debug and configuration *************************************************/

/* the following can be used to tune the delay macro (main.c) */
//#define DELAY_TUNING

/* the following can be used to control the MACRO results */
//#define MACRO_TEST

/* the following enables the output of PWM period */
//#define PWM_PERIOD_OUT_ENABLE

/* the following locks the program before system reference change, it can
be useful during debug or particularly difficult tuning; it leads to a not safe
working mode (to stop the motor a reset is needed), so enable it only if needed */
//#define CURLOOP_DEBUG

/* main loop timing debug output */
#define M_LOOP_TIM_DEB_LD3_ON	                // SET_LD3_ON
#define M_LOOP_TIM_DEB_LD3_OFF	                // SET_LD3_OFF

/* main interrupt timing debug output */
#define M_INT_TIM_DEB_LD4_ON	SET_LD4_ON
#define M_INT_TIM_DEB_LD4_OFF	SET_LD4_OFF

/* the following enables the output of waveforms; it is a normal part of the program,
so disable it only for debug purposes */
#define DEBUG_PWM

/* the following performs the phase-estimation coherency test: it is a normal part
of the program, so disable it only for debug purposes */
#define TRIP_CONTROL


/* A/D converter reference voltage and range */
#define AD_VCC_V    ( 5.0 )			// A/D Range [V]
#define AD_FSB	    ( 1024.0 )    		// A/D full scale [bit]
#define AD_ASH	    ( 6 )			// A/D alignment shift

/* Timer RD frequency */
#define TRD_FRE_HZ  ( 64000000.0 )      	// Timer RD frequency in Hz

/* hardware settings */
#ifndef RSHUNT_OHM
	#define RSHUNT_OHM	( 0.1 )		// shunt resistors [Ohm]
#endif	// ifndef RSHUNT_OHM
#ifndef RSGAIN
	#define RSGAIN		( 5.0 )		// circuit gain
#endif	// ifndef RSGAIN
#ifndef RVBUS1_OHM
	#define RVBUS1_OHM	( 47000.0 )	// split resistor 1 [Ohm]
#endif	// RVBUS1_OHM
#ifndef RVBUS2_OHM
	#define RVBUS2_OHM	( 4700.0 )	// split resistor 2 [Ohm]
#endif	// RVBUS2_OHM
#ifndef	DEADTIM_US				
	#define	DEADTIM_US	( 1.0 )         // hardware deadtimes in us

#endif	/* DEADTIM_US */

/* pwm frequency */
#ifndef PWM_FRE_CUSTOM_HZ			// desired pwm frequency in Hz
	#define PWM_FRE_CUSTOM_HZ   ( 16000 )
#endif	// PWM_FRE_CUSTOM_HZ
#if (PWM_FRE_CUSTOM_HZ < 8000)
	#define	PWM_FRE_CUS	    ( 8000 )
#elif (PWM_FRE_CUSTOM_HZ > 24000)
	#define	PWM_FRE_CUS	    ( 24000 )
#else
	#define	PWM_FRE_CUS	    ( PWM_FRE_CUSTOM_HZ )
#endif

/* sampling frequency */
#ifndef SAM_FRE_CUSTOM_HZ			// desired sampling frequency in Hz
	#define SAM_FRE_CUSTOM_HZ   ( 8000 )
#endif	// SAM_FRE_CUSTOM_HZ
#if (SAM_FRE_CUSTOM_HZ < 2000)
	#define	SAM_FRE_CUS	    ( 2000 )
#elif (SAM_FRE_CUSTOM_HZ > 8000)
	#define	SAM_FRE_CUS	    ( 8000 )
#else
	#define	SAM_FRE_CUS	    ( SAM_FRE_CUSTOM_HZ )
#endif

/* interrupt skipping ratio */
#define XYRATIO (PWM_FRE_CUS / SAM_FRE_CUS)
#if (XYRATIO<1)
	#define	TC_RATIO 	( 1 )
#elif (XYRATIO>8)
	#define	TC_RATIO 	( 8 )
#else
	#define	TC_RATIO 	( XYRATIO )
#endif
#define	INT_SKIP 		((uint8_t)(TC_RATIO - 1))

/* timing constants */
#define DEADTIM		((uint16_t)( DEADTIM_US * (TRD_FRE_HZ / 1000000.0 )))
#define	SEMIPER		((uint16_t)( 0.5 * TRD_FRE_HZ / PWM_FRE_CUS ))	// TRD pulses in 1/2 pwm period
#define PWM_PER		((uint16_t)( SEMIPER * 2 ))			// TRD pulses in 1 pwm period
#define	SEMIPERDEAD	((uint16_t)( SEMIPER + DEADTIM - 2 ))
#define SAM_PER		((uint16_t)( PWM_PER * TC_RATIO ))	        // TRD pulses in 1 sampling period
#define	PWM_PER_S	( PWM_PER / TRD_FRE_HZ )		        // actual pwm period in sec
#define	SAM_PER_S	( SAM_PER / TRD_FRE_HZ )		        // actual sampling period in sec
#define	PWM_FRE_HZ	( TRD_FRE_HZ / PWM_PER )		        // actual pwm frequency in Hz
#define	SAM_FRE_HZ	( TRD_FRE_HZ / SAM_PER )		        // actual sampling frequency in Hz
#define	SAM_FRE		((uint16_t)SAM_FRE_HZ)				// sampling frequency in Hz
#define	SAM_PER_US	((uint16_t)( 1000000.0 * SAM_PER_S ))           // sampling period in usec
#define	XMLOOP_PER_S	( 0.01 )					// desired main loop time in sec
#define NUM_INT		((uint16_t)( XMLOOP_PER_S * SAM_FRE_HZ ))       // main loop timing
#define	MLOOP_FRE_HZ	( SAM_FRE_HZ / NUM_INT )		        // actual main loop frequency in Hz
#define	MLOOP_FRE	((uint16_t)MLOOP_FRE_HZ)			// main loop frequency in Hz


/* current (Current[internal]=KI*Current[A]) */
#define KIF		( 1024.0 )	                                // a max current of 32Ampere can be managed
#define KI		((uint16_t)KIF)
#define SH_KI		( 10 )
#define AMP_KADI	( 256 )
#define SH_KADI		( 8 )
#define KADIF		(KIF * AD_VCC_V / (AD_FSB * RSHUNT_OHM * RSGAIN))
#define KADI		((uint16_t)(AMP_KADI * KADIF))	                // MUST BE SIGNED SHORT!

/* voltage (Voltage[internal]=KV*Voltage[V]) */
#define KVF		( 64.0 )		                        // a max voltage of 512Volt can be managed
#define KV		((uint16_t)KVF)
#define SH_KV		( 6 )
#define AMP_KADV	( 256 )
#define SH_KADV		( 8 )
#define KADVF		(KVF * AD_VCC_V * (RVBUS1_OHM + RVBUS2_OHM) / (RVBUS2_OHM * AD_FSB))
#define	KADV		((uint16_t)(KADVF * AMP_KADV))	                // MUST BE SIGNED SHORT!

/* resistance (Resistance[internal]=KR*Resistance[Ohm]) */
#define KRF		( 256.0 )		                        // a max resistance of 128Ohm can be managed
#define KR		((uint16_t)KRF)
#define SH_KR		( 8 )

/* inductance (Inductance[internal]=KL*Inductance[Henry]) */
#define KLF		( 16384.0 )	                                // a max inductance of 2Henry can be managed
#define KL		((uint16_t)KLF)
#define SH_KL		( 14 )

/* flux (Flux[internal]=KF*Flux[V*sec]) */
#define KFF		( 4096.0 )	                                // a max flux of 8Weber can be managed
#define KF		((uint16_t)KFF)
#define SH_KF		( 12 )

/* conversion constants */
#define KRIVF		( KIF * KRF / KVF )
#define KRIV		((uint16_t)KRIVF)	                        // Voltage[internal]=Resistance[internal]*Current[internal]/KRIV
//#define SH_KRIV			(SH_KI+SH_KR-SH_KV)
#define SH_KRIV		( 12 )			                        // 10+8-6
#define KLIFF		( KIF * KLF / KFF )
#define KLIF		((uint16_t)KLIFF)	                        // Flux[internal]=Inductance[internal]*Current[internal]/KLIF
//#define SH_KLIF			(SH_KI+SH_KL-SH_KF)
#define SH_KLIF		( 12 )			                        // 10+14-12

/* time (Time[internal]=(1/Tc)*Time[sec]=KTF*Time[sec]) */
#define	KTF		( SAM_FRE_HZ )
#define	KT		((uint16_t)KTF)


/* angles (Angle[internal]=(TWOPI/2pi)*Angle[rad]=KAF*Angle[rad]) */

/* angular speed (AngSpeed[internal]=AMP_AS*(KAF*AngSpeed[rad/sec]/KTF)) */
#define	AMP_AS		( SPEED_NSAMPLES )
#define	SH_AS		( SH_NSAMPLES )
#define	KASF		( AMP_AS * KAF / KTF )	                          // AngSpeed[internal]=KASF*AngSpeed[rad/sec]
#define AMP_STUP	( 1024 )
#define SH_STUP		( 10 )
#define AMP_TWOPIL	( AMP_AS * AMP_STUP )
#define SH_AMP_TWOPIL	( SH_AS + SH_STUP )		                  // 10+4 SH_AS+SH_STUP
#define TWOPIL		((uint32_t)( TWOPI * AMP_TWOPIL ))


/* frequency (Freq[Hertz]=AngSpeed[rad/sec]/2pi=AngSpeed[internal]/(KASF*2pi)=KOFF*AngSpeed[internal]) */
#define KOFF		( 1.0 / (KASF * TWOPIF) )
#define AMP_KOF		( 16384 )
#define SH_KOF		( 14 )
#define KOF10		((uint16_t)(AMP_KOF * KOFF * 10.0))


/* rotor speed (Speed[rpm]=60*Freq[Hertz]/cpoli=60*AngSpeed[internal]/(KFOF*cpoli)) */
#define KRPMOF		( 1.0 / (KOFF * 60.0) )
#define AMP_KRPMO	( 2048 )
#define SH_KRPMO	( 11 )
#define KRPMO		((uint16_t)(AMP_KRPMO * KRPMOF))

/* alarms */
#define	EQP_ALL		( 1 )	          // eeprom alarm code
#define	FAULT_ALL	( 2 )	          // ipm hardware alarm code
#define	TRIP_ALL	( 3 )	          // loss of phase alarm code

/* trip control */
#define	MAX_TRIP	( 20000 )

#endif /* _CONST_DEF_H */


