/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
*******************************************************************************/
/* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.   */
/*******************************************************************************
* File Name : customize.h
* Version : 2.0
* Device(s) : RL78/G14
* Tool-Chain : RL78 IAR Embedded Workbench V1.20.x
* OS : None
* H/W Platform : YRMCKITRL78G14 Rev 2.0
* Description  : custom values definition
* Operation :
* Limitations : None
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0    Production release plus rebuild for new IAR EW release
*                              modified file organisation to support object creation
******************************************************************************/

#ifndef _CUSTOMIZE_H
#define _CUSTOMIZE_H

/******************************************************************************
Macro definitions
******************************************************************************/

/* Self test exclusion if debugging */
//#define DEBUG

/* data_flash/external_eeprom/nothing selection:
   only one (or none) of the following should be uncommented
   NOTE: to use data flash, fdl_descriptor.c and the data flash library must
   be included in the project */
//#define DATA_FLASH_LINKED
#define EXT_EEPROM

/* pwm and sampling frequencies setting */
/* NOTE:
	This relation MUST be satisfied:
		PWM_FRE_CUSTOM_HZ = N * SAM_FRE_CUSTOM_HZ, where 1 <= N <= 8
*/
#define	PWM_FRE_CUSTOM_HZ   ( 24000 )	        // PWM freq [Hz], 8K < PWM_FRE_CUSTOM_HZ < 24K
#define	SAM_FRE_CUSTOM_HZ   ( 8000 )    	// Sampling freq [Hz], 2K < SAM_FRE_CUSTOM_HZ < 8K

/* high active driving signals by default; if needed uncomment the following macro */
//#define	LOW_ACTIVE_DRIVES

/* overmodulation */
//#define OVERMOD

/* Adding an offset to phase estimation (PAR_16; 65536=2pi) */
//#define PHASE_PLUS_OFFSET

/* measurement units resolution */
#define	AMP_DIV	  ( 10 )		// current parameters expressed in ampere/AMP_DIV
#define	OHM_DIV	  ( 10 )		// resistance parameters expressed in ohm/OHM_DIV
#define	HEN_DIV	  ( 10000 )   	        // resistance parameters expressed in ohm/OHM_DIV

/* MCRP07 15V motor parameters */
#define	RPM_MIN_CUSTOM	( 2000 )	//  200 < X <  5000	min speed in rpm
#define	RPM_MAX_CUSTOM	( 6000 )	//  1000 < X < 20000	max speed in rpm
#define	R_ACC_CUSTOM	( 4000 )	//  1 < X < 10000	accel. ramp in rpm/sec
#define	R_DEC_CUSTOM	( 2000 )	//  1 < X < 10000	accel. ramp in rpm/sec
#define	C_POLI_CUSTOM	( 2 )		//  1 < X <     4	polar couples number
#define	I_START_CUSTOM	( 3 )		//  0 < X <  5000	startup current in A(pk)/AMP_DIV
#define	I_MAX_CUSTOM	( 3 )		//  0 < X <  5000	max current in A(pk)/AMP_DIV
#define	R_STA_CUSTOM	( 40 )		//  0 < X <  5000	stator phase resistence in Ohm/OHM_DIV
#define	L_SYN_CUSTOM	( 30 )		//  0 < X <  5000	synchronous inductance in Henry/HEN_DIV
#define STP_TIM_CUSTOM	( 800 )		//  300 < X < 10000     startup ramp time in ms
#define	KP_CUR_CUSTOM	( 70 )		//  0 < X <  2047	K prop. current control
#define	KI_CUR_CUSTOM	( 200 )		//  0 < X <  1023	K integ. current control
#define	KP_VEL_CUSTOM	( 50 )		//  0 < X <  4095	K prop. speed control
#define	KI_VEL_CUSTOM	( 200 )		//  0 < X <  4095	K integ. speed control

#endif /* _CUSTOMIZE_H */

