/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
*******************************************************************************/
/* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.   */
/*******************************************************************************
* File Name : globdef.h
* Version : 2.0
* Device(s) : RL78/G14
* Tool-Chain : RL78 IAR Embedded Workbench V1.20.x
* OS : None
* H/W Platform : YRMCKITRL78G14 Rev 2.0
* Description  : global includes and defines
* Operation :
* Limitations : None
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.60.2012 V1.1    Production release plus rebuild for new IAR EW release
******************************************************************************/

#ifndef _GLOBDEF_H
#define _GLOBDEF_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#ifdef RL78G14_64

#include <ior5f104le.h>
#include <ior5f104le_ext.h>

#else                         /* 100 pin RL78G14 */

#include <ior5f104pj.h>
#include <ior5f104pj_ext.h>

#endif

#include <intrinsics.h>
#include "typedefine.h"
#include "mask.h"
#include "const_def.h"
#include "globvar.h"
#include "hwsetup.h"
#include "par_tab.h"

/******************************************************************************
Macro definitions
******************************************************************************/

/* intrinsic function defines */
#define DI	__disable_interrupt
#define EI	__enable_interrupt
#define HALT	__halt
#define NOP	__no_operation
#define STOP	__stop

/* Applilet defines */
#define	MD_ON	  ( 1U )
#define	MD_OFF	  ( 0U )
#define	MD_TRUE	  ( 1U )
#define	MD_FALSE  ( 0U )
#define MD_SET	  ( 1U )
#define MD_CLEAR  ( 0U )

/* TRUE/FALSE/NULL/NOP */
#ifndef TRUE
	#define TRUE    ( 1 )
#endif
#ifndef FALSE
	#define FALSE	( 0 )
#endif
#ifndef NULL
	#define NULL	( 0 )
#endif                        
#ifndef NOP
	#define NOP	{ asm("NOP"); }
#endif                        

/* Delay */
#define US1		( 5 )
#define US10		( 53 )
#define US100		( 335 )
#define DELAY(X, Y)	{ uint16_t x, y; \
			  for(x = X; x > 0; --x)  \
			  for(y = Y; y > 0; --y)  \
			  asm("NOP"); 		  }

#define DELAY1US(X)	DELAY(X, US1)
#define DELAY10US(X)	DELAY(X, US10)
#define DELAY100US(X)	DELAY(X, US100)

/* LEDs */
#define LD1_ON		(P7 & SET0)		// P70 UP_29 OUT_DL1
#define LD1_OFF		(! LD1_ON)		// P70 UP_29 OUT_DL1
#define SET_LD1_ON	{ P7 |= SET0; }		// P70 UP_29 OUT_DL1
#define SET_LD1_OFF	{ P7 &= CLR0; }		// P70 UP_29 OUT_DL1
#define LD2_ON		(P0 & SET5)		// P05 UP_31 OUT_DL2
#define LD2_OFF		(! LD2_ON)		// P05 UP_31 OUT_DL2
#define SET_LD2_ON	{ P0 |= SET5; }		// P05 UP_31 OUT_DL2
#define SET_LD2_OFF	{ P0 &= CLR5; }		// P05 UP_31 OUT_DL2
#define LD3_ON		(P7 & SET1)		// P71 UP_28 OUT_DL3
#define LD3_OFF		(! LD3_ON)		// P71 UP_28 OUT_DL3
#define SET_LD3_ON	{ P7 |= SET1; }		// P71 UP_28 OUT_DL3
#define SET_LD3_OFF	{ P7 &= CLR1; }		// P71 UP_28 OUT_DL3
#define LD4_ON		(P0 & SET6)		// P06 UP_30 OUT_DL4
#define LD4_OFF		(! LD4_ON)		// P06 UP_30 OUT_DL4
#define SET_LD4_ON	{ P0 |= SET6; }		// P06 UP_30 OUT_DL4
#define SET_LD4_OFF	{ P0 &= CLR6; }		// P06 UP_30 OUT_DL4

/* BUTTONs */
#define HW_PUL1_ON	((P5 & SET2) == 0)	// P52 UP_35 IN_PUL1
#define HW_PUL1_OFF	((P5 & SET2) != 0)	// P52 UP_35 IN_PUL1
#define HW_PUL2_ON	((P5 & SET3) == 0)	// P53 UP_36 IN_PUL2
#define HW_PUL2_OFF	((P5 & SET3) != 0)	// P53 UP_36 IN_PUL2
#define HW_PUL3_ON	((P5 & SET4) == 0)	// P54 UP_37 IN_PUL3
#define HW_PUL3_OFF	((P5 & SET4) != 0)	// P54 UP_37 IN_PUL3
#define HW_PUL4_ON	((P5 & SET5) == 0)	// P55 UP_38 IN_PUL4
#define HW_PUL4_OFF	((P5 & SET5) != 0)	// P55 UP_38 IN_PUL4

/* HALL INPUTs */
#define HALLA_UP	((P7 & SET7) != 0)	// P77 UP_22 IN_HALL_A
#define HALLA_DW	((P7 & SET7) == 0)	// P77 UP_22 IN_HALL_A
#define HALLB_UP	((P7 & SET6) != 0)	// P76 UP_23 IN_HALL_B
#define HALLB_DW	((P7 & SET6) == 0)	// P76 UP_23 IN_HALL_B
#define HALLC_UP	((P7 & SET5) != 0)	// P75 UP_24 IN_HALL_C
#define HALLC_DW	((P7 & SET5) == 0)	// P75 UP_24 IN_HALL_C

/* defines for A/D conversions */
// P27/ANI7 UP_49 IN_A7_REF
// P26/ANI6 UP_50 IN_A6_EXT
// P25/ANI5 UP_51 IN_A5_POT
// P24/ANI4 UP_52 IN_A4_TEM
// P23/ANI3 UP_53 IN_A3_VB
// P22/ANI2 UP_54 IN_A2_IU
// P21/ANI1 UP_55 IN_A1_IV
// P20/ANI0 UP_56 IN_A0_IW
#define	START_AD	{ ADCS = 1; }
#define	WAIT_AD		{ while(ADCS); }
#define	STOP_AD		{ ADCS = 0; }
#define RESULT_AD	(ADCR >> AD_ASH)
#define	SEL_CH0_AD	{ ADS = 0x00; }
#define	SEL_CH1_AD	{ ADS = 0x01; }
#define	SEL_CH2_AD	{ ADS = 0x02; }
#define	SEL_CH3_AD	{ ADS = 0x03; }
#define	SEL_CH4_AD	{ ADS = 0x04; }
#define	SEL_CH5_AD	{ ADS = 0x05; }
#define	SEL_CH6_AD	{ ADS = 0x06; }
#define	SEL_CH7_AD	{ ADS = 0x07; }
#define	SEL_CH16_AD	{ ADS = 0x10; }
#define	SEL_CH17_AD	{ ADS = 0x11; }
#define	SEL_CH18_AD	{ ADS = 0x12; }
#define	SEL_CH19_AD	{ ADS = 0x13; }

/* A/D reading */
#define SEL_IW_AD	SEL_CH0_AD
#define SEL_IV_AD	SEL_CH1_AD
#define SEL_IU_AD	SEL_CH2_AD
#define SEL_VB_AD	SEL_CH3_AD

/* defines for PWM */
// WARNING: PWM OUTPUT SIGNALS
// P17/TO02    UP_39 OUT_TP2PWM
// P16/TO01    UP_40 OUT_TP1PWM
// P15/TRDIOB0 UP_41 OUT_PWM_UH
// P14/TRDIOD0 UP_42 OUT_PWM_UL
// P13/TRDIOA1 UP_43 OUT_PWM_VH
// P12/TRDIOB1 UP_44 OUT_PWM_WH
// P11/TRDIOC1 UP_45 OUT_PWM_VL
// P10/TRDIOD1 UP_46 OUT_PWM_WL

#define SET_PWM(U, V, W)  { TRDMR = 0x00; TRDGRD0 = U; TRDGRC1 = V; TRDGRD1 = W; TRDMR = 0xE0; }
#define RES_PWM_OUT	  { TRDOER1 |= 0xFA; }
#define SET_PWM_OUT	  { TRDOER1 &= 0x05; P1 &= 0xC0; }

#ifdef LOW_ACTIVE_DRIVES
#define SET_ALL_OFF	  { P1 |= 0x3F; RES_PWM_OUT }
#define SET_LOW_ON	  { P1 &= 0xEC; RES_PWM_OUT }

#else	/* else of LOW_ACTIVE_DRIVES means HIGH_ACTIVE_DRIVES */
#define SET_ALL_OFF	  { P1 &= 0xC0; RES_PWM_OUT }
#define SET_LOW_ON	  { P1 |= 0x13; RES_PWM_OUT }

#endif	/* LOW_ACTIVE_DRIVES */

/* define for drive the LIM_OFF transistor */
#define	SET_USB_PWR_ON	  { P13 |= SET0; }	// P130 UP_57 OUT_LIM_OFF

#endif /* _GLOBDEF_H */

