
/******************************************************************************
* DISCLAIMER

* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized.

* This software is owned by Renesas Electronics Corporation and is protected under 
* all applicable laws, including copyright laws.

* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES 
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, 
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
* PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY 
* DISCLAIMED.

* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES 
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS 
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

* Renesas reserves the right, without notice, to make changes to this 
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and 
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer

******************************************************************************/
/******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : hwsetup.c
* Version      : 2.0
* Device(s)    : RL78G14
* Tool-Chain   : RL78 IAR Embedded Workbench V1.20.x
* H/W Platform : YRMCKITRL78G14 V2.0
* Description  : Hardware setup file - Source build
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0    Production release plus rebuild for new IAR EW release
*                              modified file organisation to support object creation
*******************************************************************************/



/******************************************************************************
* uP configuration description (64 pins version)
*******************************************************************************
*
*  1 P120/ANI19   NU (IN)
*  2 P43          NU (IN)
*  3 P42          NU (IN)
*  4 P41          NU (IN)
*  5 P40/TOOL0    EMUL
*  6 RESET        RESET
*  7 P124         NU (IN)
*  8 P123         NU (IN)
*  9 P137/INTP0   IN_OVCUR
* 10 P122/X2      NU (OSCI_X2)
* 11 P121/X1      NU (OSCI_X1)
* 12 REGC
* 13 VSS
* 14 EVSS
* 15 VDD
* 16 EVDD
* >
* 17 P60/SCLA0    OUT_EEP_SCL
* 18 P61/SDAA0    IO_EEP_SDA
* 19 P62          NU (IN)
* 20 P63          IN_EXT_SUP
* 21 P31/TO03     OUT_TP3PWM
* 22 P77          IN_HALL_A
* 23 P76          IN_HALL_B
* 24 P75          IN_HALL_C
* 25 P74          NU (IN)
* 26 P73          NU (IN)
* 27 P72          NU (IN)
* 28 P71          OUT_DL3
* 29 P70          OUT_DL1
* 30 P06          OUT_DL4
* 31 P05          OUT_DL2
* 32 P30          NU (OUT)
* >
* 33 P50/RXD0     IN_RXD
* 34 P51/TXD0     OUT_TXD
* 35 P52          IN_PUL1
* 36 P53          IN_PUL2
* 37 P54          IN_PUL3
* 38 P55          IN_PUL4
* 39 P17/TO02     OUT_TP2PWM
* 40 P16/TO01     OUT_TP1PWM
* 41 P15/TRDIOB0  OUT_PWM_UH
* 42 P14/TRDIOD0  OUT_PWM_UL
* 43 P13/TRDIOA1  OUT_PWM_VH
* 44 P12/TRDIOB1  OUT_PWM_WH
* 45 P11/TRDIOC1  OUT_PWM_VL
* 46 P10/TRDIOD1  OUT_PWM_WL
* 47 P146         NU (IN)
* 48 P147/ANI18   NU (IN)
* >
* 49 P27/ANI7     IN_A7_REF
* 50 P26/ANI6     IN_A6_EXT
* 51 P25/ANI5     IN_A5_POT
* 52 P24/ANI4     IN_A4_TEM
* 53 P23/ANI3     IN_A3_VB
* 54 P22/ANI2     IN_A2_IU
* 55 P21/ANI1     IN_A1_IV
* 56 P20/ANI0     IN_A0_IW
* 57 P130         OUT_LIM_OFF
* 58 P04          NU (OUT)
* 59 P03/RXD1     IN_RXD1
* 60 P02/TXD1     OUT_TXD1
* 61 P01/TRGCLKB  IN_ENC_B
* 62 P00/TRGCLKA  IN_ENC_A
* 63 P141         IN_ENC_Z
* 64 P140         NU (IN)
*
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "globdef.h"
#include "hwsetup.h"

/******************************************************************************
Private global variables and constants
******************************************************************************/

/* Set option bytes */
#pragma location = "OPTBYTE"
__root const UCHAR opbyte0 = 0xEFU;
#pragma location = "OPTBYTE"
__root const UCHAR opbyte1 = 0x73U;
#pragma location = "OPTBYTE"
__root const UCHAR opbyte2 = 0xF8U; 	// Set to 64MHZ 0xE8U to have TRD running @32MHz
#pragma location = "OPTBYTE"
__root const UCHAR opbyte3 = 0x85U;

/* Set security ID */
#pragma location = "SECUID"
__root const UCHAR secuid[10] =
	{0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U};


/******************************************************************************
global variables and functions
******************************************************************************/
uint8_t resetflag;


/******************************************************************************
Private functions
******************************************************************************/

/******************************************************************************
* Function Name: ReadResetSource
* Description  : reading of reset source
* Arguments    : none
* Return Value : none
******************************************************************************/
void ReadResetSource(void)
{

	resetflag = RESF;

}	/* end of function ReadResetSource() */

/******************************************************************************
* Function Name: systeminit
* Description  : system init
* Arguments    : none
* Return Value : none
******************************************************************************/
static void systeminit(void)
{       
	CRC0CTL = 0x00U;
	IAWCTL = 0x00U;
        RPECTL = 0x80;

}	/* end of function systeminit() */

/******************************************************************************
* Function Name: CLOCK_Init
* Description  : internal clock init
* Arguments    : none
* Return Value : none
******************************************************************************/
static void CLOCK_Init(void)
{
	/* Set fMX */
	// external oscillators not used: X1 and XT1 are input ports
	CMC = _00_CG_HISYS_PORT | _00_CG_SUB_PORT | _00_CG_SYSOSC_DEFAULT | _00_CG_SUBMODE_DEFAULT;
	MSTOP = 1U;			// X1 oscillator stopped

	/* Set fMAIN */
	MCM0 = 0U;			// select internal high speed oscillator fIH for fMAIN

	/* Set fSUB */
	XTSTOP = 1U;		        // XT1 oscillator stopped
	OSMC = _10_CG_RTC_CLK_FIL;      // select internal low speed osc. for RTC and interval timer

	/* Set fCLK */
	CSS = 0U;			// select fMAIN for CPU/peripherals hw clock fCLK

	/* Set fIH */
	HIOSTOP = 0U;		        // internal high speed oscillator operating


	/* peripheral clock supply (at reset all stopped) */
	PER0 = 0x00;
	//RTCEN = 0;				// PER0.7 RTC not enabled
	//ADCEN = 0;				// PER0.5 ADC not enabled
	//IICA0EN = 0;				// PER0.4 IICA0 not enabled
	//SAU1EN = 0;				// PER0.3 SAU1 not enabled
	//SAU0EN = 0;				// PER0.2 SAU0 not enabled
	//TAU0EN = 0;				// PER0.0 TAU0 not enabled
        
	PER1 = 0x00;
	//TRGEN = 0;				// PER1.6 TRG not enabled
	//TRD0EN = 0;				// PER1.4 TRD0 not enabled
	//DTCEN = 0;				// PER1.3 DTC not enabled
	//TRJ0EN = 0;				// PER1.0 TRJ0 not enabled

}	/* end of function CLOCK_Init() */


/******************************************************************************
* Function Name: MC_IniPWM
* Description  : pwm initialization
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_IniPWM(void)
{

	/* wake up peripheral */
	TRD0EN = 1;	          // PER1.4 Peripheral Enable Register 1
	DELAY1US(10)

	/* disable outputs */
#ifdef LOW_ACTIVE_DRIVES
	P1 |= 0x3F;		  // output latch setting
        
#else	/* else of LOW_ACTIVE_DRIVES means HIGH_ACTIVE_DRIVES */
	
        P1 &= 0xC0;		  // output latch setting
#endif	/* LOW_ACTIVE_DRIVES */
        
	PM1 = 0x00;		  // port mode 0=output, 1=input
	
        TRDOER1 = 0xFF;		  // Timer RD Output Master Enable Register 1 (reset 0xFF)
	//ED1 = 1;		  // TRDOER1.7 TRDIOD1 output 0-> enabled, 1-> disabled
	//EC1 = 1;		  // TRDOER1.6 TRDIOC1 output 0-> enabled, 1-> disabled
	//EB1 = 1;		  // TRDOER1.5 TRDIOB1 output 0-> enabled, 1-> disabled
	//EA1 = 1;		  // TRDOER1.4 TRDIOA1 output 0-> enabled, 1-> disabled
	//ED0 = 1;		  // TRDOER1.3 TRDIOD0 output 0-> enabled, 1-> disabled
	//EC0 = 1;		  // TRDOER1.2 TRDIOC0 output 0-> enabled, 1-> disabled
	//EB0 = 1;		  // TRDOER1.1 TRDIOB0 output 0-> enabled, 1-> disabled
	//EA0 = 1;		  // TRDOER1.0 TRDIOA0 output 0-> enabled, 1-> disabled

	/* disable interrupts */
	TRDMK1 = 1;		  // MK2H.2 Disable TRD1 interrupt
	TRDMK0 = 1;		  // MK2H.1 Disable TRD0 interrupt
	TRDIF1 = 0;		  // IF2H.2 Reset TRD1 interrupt flag
	TRDIF0 = 0;		  // IF2H.1 Reset TRD0 interrupt flag

	/* stop timers */
	TRDSTR = 0x0C;		  // Timer RD Start Register (reset 0x0C)
	//CSEL1 = 1;		  // TRDSTR.3 at compare match with TRDGRA1: 0-> stop, 1-> continue
	//CSEL0 = 1;		  // TRDSTR.2 at compare match with TRDGRA0: 0-> stop, 1-> continue
	//TSTART1 = 0;		  // TRDSTR.1 1-> TRD1 start counting, 0-> stop
	//TSTART0 = 0;		  // TRDSTR.0 1-> TRD0 start counting, 0-> stop

#ifdef LOW_ACTIVE_DRIVES

	/* re-set  CMD0,1 */
	TRDFCR = 0x80;		  // Timer RD Function Control Register (reset 0x80)
	//PWM3 = 1;		  // TRDFCR.7 0-> PWM3, 1-> other than PWM3
	//STCLK = 0;		  // TRDFCR.6 external clock input 0-> disabled, 1-> enabled
	//OSL1 = 0;		  // TRDFCR.3 Counter-phase output level select 0-> low active, 1-> high active
	//OSL0 = 0;		  // TRDFCR.2 Phase output level select 0-> low active, 1-> high active
	//CMD1 = 0;		  // TRDFCR.1 combination mode sel 1-> complementary PWM, 0-> other than comp. PWM
	//CMD0 = 0;		  // TRDFCR.0 combination mode sel (in comp. PWM 0-> transfer at through, 1-> transfer at crest)

	/* set again TRDFCR for complementary PWM, internal clock, low active, transfer at trough */
	TRDFCR = 0x82;		  // Timer RD Function Control Register (reset 0x80)
	//PWM3 = 1;		  // TRDFCR.7 0-> PWM3, 1-> other than PWM3
	//STCLK = 0;		  // TRDFCR.6 external clock input 0-> disabled, 1-> enabled
	//OSL1 = 0;		  // TRDFCR.3 Counter-phase output level select 0-> low active, 1-> high active
	//OSL0 = 0;		  // TRDFCR.2 Phase output level select 0-> low active, 1-> high active
	//CMD1 = 1;		  // TRDFCR.1 combination mode sel 1-> complementary PWM, 0-> other than comp. PWM
	//CMD0 = 0;		  // TRDFCR.0 combination mode sel (in comp. PWM 0-> transfer at trough, 1-> transfer at crest)

#else	/* else HIGH_ACTIVE_DRIVE */

	/* re-set  CMD0,1 */
	TRDFCR = 0x8C;		  // Timer RD Function Control Register (reset 0x80)
	//PWM3 = 1;		  // TRDFCR.7 0-> PWM3, 1-> other than PWM3
	//STCLK = 0;		  // TRDFCR.6 external clock input 0-> disabled, 1-> enabled
	//OSL1 = 1;		  // TRDFCR.3 Counter-phase output level select 0-> low active, 1-> high active
	//OSL0 = 1;		  // TRDFCR.2 Phase output level select 0-> low active, 1-> high active
	//CMD1 = 0;		  // TRDFCR.1 combination mode sel 1-> complementary PWM, 0-> other than comp. PWM
	//CMD0 = 0;		  // TRDFCR.0 combination mode sel (in comp. PWM 0-> transfer at through, 1-> transfer at crest)

	/* set again TRDFCR for complementary PWM, internal clock, low active, transfer at trough */
	TRDFCR = 0x8E;		  // Timer RD Function Control Register (reset 0x80)
	//PWM3 = 1;		  // TRDFCR.7 0-> PWM3, 1-> other than PWM3
	//STCLK = 0;		  // TRDFCR.6 external clock input 0-> disabled, 1-> enabled
	//OSL1 = 1;		  // TRDFCR.3 Counter-phase output level select 0-> low active, 1-> high active
	//OSL0 = 1;		  // TRDFCR.2 Phase output level select 0-> low active, 1-> high active
	//CMD1 = 1;		  // TRDFCR.1 combination mode sel 1-> complementary PWM, 0-> other than comp. PWM
	//CMD0 = 0;		  // TRDFCR.0 combination mode sel (in comp. PWM 0-> transfer at trough, 1-> transfer at crest)

#endif	/* LOW_ACTIVE_DRIVES */

	/* disable outputs again */
	TRDOER1 = 0xFF;		  // Timer RD Output Master Enable Register 1 (reset 0xFF)

	/* select clock for TRD0 */
	TRDCR0 = 0x00;		  // Timer RD Control Register 0 (reset 0x00)
	//CCLR2 = 0;		  // TRDCR0.7 TRD0 counter clear select (comp. PWM: set to 0)
	//CCLR1 = 0;		  // TRDCR0.6 TRD0 counter clear select (comp. PWM: set to 0)
	//CCLR0 = 0;		  // TRDCR0.5 TRD0 counter clear select (comp. PWM: set to 0)
	//CKEG1 = 0;		  // TRDCR0.4 ext. clock edge sel. (00 -> rising)
	//CKEG0 = 0;		  // TRDCR0.3 ext. clock edge sel.
	//TCK2 = 0;		  // TRDCR0.2 clock source sel. (000 -> fIH)
	//TCK1 = 0;		  // TRDCR0.1 clock source sel.
	//TCK0 = 0;		  // TRDCR0.0 clock source sel.

	/* select clock for TRD1 */
	TRDCR1 = 0x00;		  // Timer RD Control Register 1 (reset 0x00)
	//CCLR2 = 0;		  // TRDCR1.7 TRD1 counter clear select (comp. PWM: set to 0)
	//CCLR1 = 0;		  // TRDCR1.6 TRD1 counter clear select (comp. PWM: set to 0)
	//CCLR0 = 0;		  // TRDCR1.5 TRD1 counter clear select (comp. PWM: set to 0)
	//CKEG1 = 0;		  // TRDCR1.4 ext. clock edge sel. (00 -> rising)
	//CKEG0 = 0;		  // TRDCR1.3 ext. clock edge sel.
	//TCK2 = 0;		  // TRDCR1.2 clock source sel. (000 -> fIH)
	//TCK1 = 0;		  // TRDCR1.1 clock source sel.
	//TCK0 = 0;		  // TRDCR1.0 clock source sel.

	/* select PWM function for output pins */
	TRDPMR = 0x77;		  // Timer RD PWM Function Select Register (reset 0x00)
	//PWMD1 = 1;		  // TRDPMR.6 TRDIOD1: 0-> IC or OC, 1-> PWM
	//PWMC1 = 1;		  // TRDPMR.5 TRDIOC1: 0-> IC or OC, 1-> PWM
	//PWMB1 = 1;		  // TRDPMR.4 TRDIOB1: 0-> IC or OC, 1-> PWM
	//PWMD0 = 1;		  // TRDPMR.2 TRDIOD0: 0-> IC or OC, 1-> PWM
	//PWMC0 = 1;		  // TRDPMR.1 TRDIOC0: 0-> IC or OC, 1-> PWM
	//PWMB0 = 1;		  // TRDPMR.0 TRDIOB0: 0-> IC or OC, 1-> PWM

	/* select initial output level */
	TRDOCR = 0x00;		  // Timer RD Output Control Register (reset 0x00)
	//TOD1 = 0;		  // TRDOCR.7 TRDIOD1 initial output level select (PWM: 0-> not active level, 1-> active level)
	//TOC1 = 0;		  // TRDOCR.6 TRDIOC1 initial output level select (PWM: 0-> not active level, 1-> active level)
	//TOB1 = 0;		  // TRDOCR.5 TRDIOB1 initial output level select (PWM: 0-> not active level, 1-> active level)
	//TOA1 = 0;		  // TRDOCR.4 TRDIOA1 initial output level select (PWM: 0-> not active level, 1-> active level)
	//TOD0 = 0;		  // TRDOCR.3 TRDIOD0 initial output level select (PWM: 0-> not active level, 1-> active level)
	//TOC0 = 0;		  // TRDOCR.2 TRDIOC0 initial output level select (PWM: 0-> not active level, 1-> active level)
	//TOB0 = 0;		  // TRDOCR.1 TRDIOB0 initial output level select (PWM: 0-> not active level, 1-> active level)
	//TOA0 = 0;		  // TRDOCR.0 TRDIOA0 initial output level select (PWM: 0-> not active level, 1-> active level)

#ifdef LOW_ACTIVE_DRIVES

	/* select output level */
	TRDPOCR0 = 0x00;	  // Timer RD PWM Function Output Level Control Register 0 (reset 0x00)
	//POLD = 0;		  // TRDPOCR0.2 TRDIOD0 PWM output level: 0-> low active, 1-> high active
	//POLC = 0;		  // TRDPOCR0.1 TRDIOC0 PWM output level: 0-> low active, 1-> high active
	//POLB = 0;		  // TRDPOCR0.0 TRDIOB0 PWM output level: 0-> low active, 1-> high active
        
	TRDPOCR1 = 0x00;	  // Timer RD PWM Function Output Level Control Register 1 (reset 0x00)
	//POLD = 0;		  // TRDPOCR1.2 TRDIOD1 PWM output level: 0-> low active, 1-> high active
	//POLC = 0;		  // TRDPOCR1.1 TRDIOC1 PWM output level: 0-> low active, 1-> high active
	//POLB = 0;		  // TRDPOCR1.0 TRDIOB1 PWM output level: 0-> low active, 1-> high active

#else	/* else HIGH_ACTIVE_DRIVE */

	/* select output level */
	TRDPOCR0 = 0x07;	  // Timer RD PWM Function Output Level Control Register 0 (reset 0x00)
	//POLD = 1;		  // TRDPOCR0.2 TRDIOD0 PWM output level: 0-> low active, 1-> high active
	//POLC = 1;		  // TRDPOCR0.1 TRDIOC0 PWM output level: 0-> low active, 1-> high active
	//POLB = 1;		  // TRDPOCR0.0 TRDIOB0 PWM output level: 0-> low active, 1-> high active
	TRDPOCR1 = 0x07;	  // Timer RD PWM Function Output Level Control Register 1 (reset 0x00)
	//POLD = 1;		  // TRDPOCR1.2 TRDIOD1 PWM output level: 0-> low active, 1-> high active
	//POLC = 1;		  // TRDPOCR1.1 TRDIOC1 PWM output level: 0-> low active, 1-> high active
	//POLB = 1;		  // TRDPOCR1.0 TRDIOB1 PWM output level: 0-> low active, 1-> high active

#endif	/* LOW_ACTIVE_DRIVES */

	/* set buffers as general registers to allow writing */
	TRDMR = 0x00;		  // Timer RD Mode Register (reset 0x00)
	//BFD1 = 0;		  // TRDMR.7 TRDGRD1 0-> general register, 1-> buffer for TRDGRB1
	//BFC1 = 0;		  // TRDMR.6 TRDGRC1 0-> general register, 1-> buffer for TRDGRA1
	//BFD0 = 0;		  // TRDMR.5 TRDGRD0 0-> general register, 1-> buffer for TRDGRB0
	//BFC0 = 0;		  // TRDMR.4 TRDGRC0 0-> general register, 1-> buffer for TRDGRA0
	//SYNC = 0;		  // TRDMR.0 0-> TRD0 and TRD1 work independently, 1-> they work synchronously

	/* write values to buffers and registers */
	TRD0 = DEADTIM;		  // Timer RD Counter 0 (reset 0x0000)
	TRD1 = 0x0000;		  // Timer RD Counter 1 (reset 0x0000)
	TRDGRA0 = SEMIPERDEAD;	  // Timer RD General Register A0 (reset 0xFFFF)
	TRDGRB0 = SEMIPERDEAD;	  // Timer RD General Register B0 (reset 0xFFFF)
	TRDGRD0 = SEMIPERDEAD;	  // Timer RD General Register D0 (reset 0xFFFF)
	TRDGRA1 = SEMIPERDEAD;	  // Timer RD General Register A1 (reset 0xFFFF)
	TRDGRB1 = SEMIPERDEAD;	  // Timer RD General Register B1 (reset 0xFFFF)
	TRDGRC1 = SEMIPERDEAD;	  // Timer RD General Register C1 (reset 0xFFFF)
	TRDGRD1 = SEMIPERDEAD;	  // Timer RD General Register D1 (reset 0xFFFF)

	/* set buffers as buffers */
	TRDMR = 0xE0;		  // Timer RD Mode Register (reset 0x00)
	//BFD1 = 1;		  // TRDMR.7 TRDGRD1 0-> general register, 1-> buffer for TRDGRB1
	//BFC1 = 1;		  // TRDMR.6 TRDGRC1 0-> general register, 1-> buffer for TRDGRA1
	//BFD0 = 1;		  // TRDMR.5 TRDGRD0 0-> general register, 1-> buffer for TRDGRB0
	//BFC0 = 0;		  // TRDMR.4 TRDGRC0 0-> general register, 1-> buffer for TRDGRA0
	//SYNC = 0;		  // TRDMR.0 0-> TRD0 and TRD1 work independently, 1-> they work synchronously

#ifdef LOW_ACTIVE_DRIVES

	/* forced cutoff control */
	TRDDF0 = 0xFF;		  // Timer RD Digital Filter Function Select Register 0 (reset 0x00)
	//DFCK1 = 1;		  // TRDDF0.7 (PWM: TRDIOA pin pulse forced cutoff control, 11-> high output)
	//DFCK0 = 1;		  // TRDDF0.6 (PWM: TRDIOA pin pulse forced cutoff control, 11-> high output)
	//PENB1 = 1;		  // TRDDF0.5 (PWM: TRDIOB pin pulse forced cutoff control, 11-> high output)
	//PENB0 = 1;		  // TRDDF0.4 (PWM: TRDIOB pin pulse forced cutoff control, 11-> high output)
	//DFD = 1;		  // TRDDF0.3 (PWM: TRDIOC pin pulse forced cutoff control, 11-> high output)
	//DFC = 1;		  // TRDDF0.2 (PWM: TRDIOC pin pulse forced cutoff control, 11-> high output)
	//DFB = 1;		  // TRDDF0.1 (PWM: TRDIOD pin pulse forced cutoff control, 11-> high output)
	//DFA = 1;		  // TRDDF0.0 (PWM: TRDIOD pin pulse forced cutoff control, 11-> high output)
        
	TRDDF1 = 0xFF;		  // Timer RD Digital Filter Function Select Register 1 (reset 0x00)
	//DFCK1 = 1;		  // TRDDF1.7 (PWM: TRDIOA pin pulse forced cutoff control, 11-> high output)
	//DFCK0 = 1;		  // TRDDF1.6 (PWM: TRDIOA pin pulse forced cutoff control, 11-> high output)
	//PENB1 = 1;		  // TRDDF1.5 (PWM: TRDIOB pin pulse forced cutoff control, 11-> high output)
	//PENB0 = 1;		  // TRDDF1.4 (PWM: TRDIOB pin pulse forced cutoff control, 11-> high output)
	//DFD = 1;		  // TRDDF1.3 (PWM: TRDIOC pin pulse forced cutoff control, 11-> high output)
	//DFC = 1;		  // TRDDF1.2 (PWM: TRDIOC pin pulse forced cutoff control, 11-> high output)
	//DFB = 1;		  // TRDDF1.1 (PWM: TRDIOD pin pulse forced cutoff control, 11-> high output)
	//DFA = 1;		  // TRDDF1.0 (PWM: TRDIOD pin pulse forced cutoff control, 11-> high output)

#else	/* else HIGH_ACTIVE_DRIVE */

	/* forced cutoff control */
	TRDDF0 = 0xFF;		  // Timer RD Digital Filter Function Select Register 0 (reset 0x00)
	//DFCK1 = 1;		  // TRDDF0.7 (PWM: TRDIOA pin pulse forced cutoff control, 10-> low output)
	//DFCK0 = 1;		  // TRDDF0.6 (PWM: TRDIOA pin pulse forced cutoff control, 10-> low output)
	//PENB1 = 1;		  // TRDDF0.5 (PWM: TRDIOB pin pulse forced cutoff control, 10-> low output)
	//PENB0 = 1;		  // TRDDF0.4 (PWM: TRDIOB pin pulse forced cutoff control, 10-> low output)
	//DFD = 1;		  // TRDDF0.3 (PWM: TRDIOC pin pulse forced cutoff control, 10-> low output)
	//DFC = 1;		  // TRDDF0.2 (PWM: TRDIOC pin pulse forced cutoff control, 10-> low output)
	//DFB = 1;		  // TRDDF0.1 (PWM: TRDIOD pin pulse forced cutoff control, 10-> low output)
	//DFA = 1;		  // TRDDF0.0 (PWM: TRDIOD pin pulse forced cutoff control, 10-> low output)
        
	TRDDF1 = 0xFF;		  // Timer RD Digital Filter Function Select Register 1 (reset 0x00)
	//DFCK1 = 1;		  // TRDDF1.7 (PWM: TRDIOA pin pulse forced cutoff control, 10-> low output)
	//DFCK0 = 1;		  // TRDDF1.6 (PWM: TRDIOA pin pulse forced cutoff control, 10-> low output)
	//PENB1 = 1;		  // TRDDF1.5 (PWM: TRDIOB pin pulse forced cutoff control, 10-> low output)
	//PENB0 = 1;		  // TRDDF1.4 (PWM: TRDIOB pin pulse forced cutoff control, 10-> low output)
	//DFD = 1;		  // TRDDF1.3 (PWM: TRDIOC pin pulse forced cutoff control, 10-> low output)
	//DFC = 1;		  // TRDDF1.2 (PWM: TRDIOC pin pulse forced cutoff control, 10-> low output)
	//DFB = 1;		  // TRDDF1.1 (PWM: TRDIOD pin pulse forced cutoff control, 10-> low output)
	//DFA = 1;		  // TRDDF1.0 (PWM: TRDIOD pin pulse forced cutoff control, 10-> low output)

#endif	/* LOW_ACTIVE_DRIVES */
        
        
        
/****************************************************************************** 
 Set ELC for Interrupt skipping - ELC set for TRD1 underflow 
*******************************************************************************/        
	ELSELR12 = 0x04;


/****************************************************************************** 
 Set TRJ0 for Interrupt skipping - TRJ0 set to Event Count, trigger from ELC
*******************************************************************************/
        TRJ0EN = 1;			// PER1.0 TRJ0 enabled
	TRJCR0 = 0x00;		        // timer stopped
	TRJIOC0 = 0x00;		        // output disabled etc.
	TRJMR0 = 0x50;		        // reset 0x00
	//TCK2 = 1;			TRJMR0.6 101 -> ELC event
	//TCK1 = 0;			TRJMR0.5
	//TCK0 = 1;			TRJMR0.4
	//TEDGPL = 0;		        TRJMR0.3 0 -> one edge
	//TMOD2 = 0;		        TRJMR0.2 000 -> timer mode
	//TMOD1 = 0;		        TRJMR0.1
	//TMOD0 = 0;		         TRJMR0.0


}	/* end of function MC_IniPWM() */




/******************************************************************************
* Function Name: hwsetup
* Description  : hardware setup
* Arguments    : none
* Return Value : none
******************************************************************************/
static void hwsetup(void)
{
	/* port 0 */
	// P06          UP_30 OUT_DL4
	// P05          UP_31 OUT_DL2
	// P04          UP_58 NU (OUT)
	// P03/RXD1     UP_59 IN_RXD1
	// P02/TXD1     UP_60 OUT_TXD1
	// P01/TRGCLKB  UP_61 IN_ENC_B
	// P00/TRGCLKA  UP_62 IN_ENC_A
	P0   = 0x00;	// output latch setting
	PM0  = 0x8B;	// port mode 0=output, 1=input
	PU0  = 0x00;	// pull-up 0=off, 1=on
	PIM0 = 0x00;	// input buffer 0=normal, 1=TTL
	POM0 = 0x00;	// output mode 0=normal, 1=open drain
	PMC0 = 0xFF;	// port mode control 0=digital, 1=analog

	/* port 1 (WARNING: PWM OUTPUT SIGNALS) */
	// P17/TO02     UP_39 OUT_TP2PWM
	// P16/TO01     UP_40 OUT_TP1PWM
	// P15/TRDIOB0  UP_41 OUT_PWM_UH
	// P14/TRDIOD0  UP_42 OUT_PWM_UL
	// P13/TRDIOA1  UP_43 OUT_PWM_VH
	// P12/TRDIOB1  UP_44 OUT_PWM_WH
	// P11/TRDIOC1  UP_45 OUT_PWM_VL
	// P10/TRDIOD1  UP_46 OUT_PWM_WL
        
#ifdef LOW_ACTIVE_DRIVES
	P1   = 0x3F;	// output latch setting
	PM1  = 0x00;	// port mode 0=output, 1=input
	PU1  = 0x00;	// pull-up 0=off, 1=on
	PIM1 = 0x00;	// input buffer 0=normal, 1=TTL
	POM1 = 0x00;	// output mode 0=normal, 1=open drain
#else	// else of LOW_ACTIVE_DRIVES means HIGH_ACTIVE_DRIVES
	P1   = 0x00;	// output latch setting
	PM1  = 0x00;	// port mode 0=output, 1=input
	PU1  = 0x00;	// pull-up 0=off, 1=on
	PIM1 = 0x00;	// input buffer 0=normal, 1=TTL
	POM1 = 0x00;	// output mode 0=normal, 1=open drain
#endif	// LOW_ACTIVE_DRIVES

	/* port 2 */
	// P27/ANI7     UP_49 IN_A7_REF
	// P26/ANI6     UP_50 IN_A6_EXT
	// P25/ANI5     UP_51 IN_A5_POT
	// P24/ANI4     UP_52 IN_A4_TEM
	// P23/ANI3     UP_53 IN_A3_VB
	// P22/ANI2     UP_54 IN_A2_IU
	// P21/ANI1     UP_55 IN_A1_IV
	// P20/ANI0     UP_56 IN_A0_IW
	P2   = 0x00;	// output latch setting
	PM2  = 0xFF;	// port mode 0=output, 1=input
	ADPC = 0x0F;	// A/D port configuration 0x0F -> all pins analog inputs

	/* port 3 */
	// P31/TO03     UP_21 OUT_TP3PWM
	// P30          UP_32 NU (OUT)
	P3   = 0x00;	// output latch setting
	PM3  = 0xFC;	// port mode 0=output, 1=input
	PU3  = 0x00;	// pull-up 0=off, 1=on
	PIM3 = 0x00;	// input buffer 0=normal, 1=TTL

	/* port 4 */
	// P43          UP__2 NU (IN)
	// P42          UP__3 NU (IN)
	// P41          UP__4 NU (IN)
	// P40/TOOL0    UP__5 EMUL
	P4  = 0x00;	// output latch setting
	PM4 = 0xFF;	// port mode 0=output, 1=input
	PU4 = 0x00;	// pull-up 0=off, 1=on

	/* port 5 */
	// P55          UP_38 IN_PUL4
	// P54          UP_37 IN_PUL3
	// P53          UP_36 IN_PUL2
	// P52          UP_35 IN_PUL1
	// P51/TXD0     UP_34 OUT_TXD
	// P50/RXD0     UP_33 IN_RXD
	P5   = 0x02;	// output latch setting
	PM5  = 0xFD;	// port mode 0=output, 1=input
	PU5  = 0x00;	// pull-up 0=off, 1=on
	PIM5 = 0x00;	// input buffer 0=normal, 1=TTL
	POM5 = 0x00;	// output mode 0=normal, 1=open drain

	/* port 6 */
	// P63          UP_20 IN_EXT_SUP
	// P62          UP_19 NU (IN)
	// P61/SDAA0    UP_18 IO_EEP_SDA
	// P60/SCLA0    UP_17 OUT_EEP_SCL
	P6  = 0x01;	// output latch setting
	PM6 = 0xFE;	// port mode 0=output, 1=input

	/* port 7 */
	// P77          UP_22 IN_HALL_A
	// P76          UP_23 IN_HALL_B
	// P75          UP_24 IN_HALL_C
	// P74          UP_25 NU (IN)
	// P73          UP_26 NU (IN)
	// P72          UP_27 NU (IN)
	// P71          UP_28 OUT_DL3
	// P70          UP_29 OUT_DL1
	P7   = 0x00;	// output latch setting
	PM7  = 0xFC;	// port mode 0=output, 1=input
	PU7  = 0x00;	// pull-up 0=off, 1=on
	POM7 = 0x00;	// output mode 0=normal, 1=open drain

	/* port 12 */
	// P124         UP__7 NU (IN)
	// P123         UP__8 NU (IN)
	// P122/X2      UP_10 NU (OSCI_X2)
	// P121/X1      UP_11 NU (OSCI_X1)
	// P120/ANI19   UP__1 NU (IN)
	P12   = 0x00;	// output latch setting
	PM12  = 0xFF;	// port mode 0=output, 1=input
	PU12  = 0x00;	// pull-up 0=off, 1=on
	PMC12 = 0xFF;	// port mode control 0=digital, 1=analog

	/* port 13 */
	// P137/INTP0   UP__9 IN_OVCUR
	// P130         UP_57 OUT_LIM_OFF
	P13 = 0x00;	// output latch setting

	/* port 14 */
	// P147/ANI18   UP_48 NU (IN)
	// P146         UP_47 NU (IN)
	// P141         UP_63 IN_ENC_Z
	// P140         UP_64 NU (IN)
	P14   = 0x00;	// output latch setting
	PM14  = 0xFF;	// port mode 0=output, 1=input
	PU14  = 0x00;	// pull-up 0=off, 1=on
	PMC14 = 0xFF;	// port mode control 0=digital, 1=analog
        
        
/* 16 Bit Timer Array Unit settings  
   Channel 1 used for hardware system clock measurement */
    TAU0EN = 1U;        /* enable clock for TAU0 */
    TPS0   = 0x0000U;   /* CKS0 = fclk (32MHz) */
    TMR01  = 0x0104U;   /* Select CK00 as Count Clock (CKS000, CKS001 = 00,  CCS00 = 0) */
                        /* Operate in Single operation function (MASTER00 = 0) */
                        /* Use software trigger start (STS000, STS001, STS002 = 0,0,0) */
                        /* select capture Timer Mode (MD001, MD002, MD003 = 1,0,0) */
                        /* Falling edge */
                        /* Interrupt is not generated when counting started (MD000 = 0) */
    TMIF01 = 0U;        /* Clear Interrupt flag */
    TMMK01 = 1U;        /* mask Timer 01 interrupt */
    TIS0   = 0x04U;     /* CH1 input clock selection => LS Oscillator (15KHz +/- 10%) */
    TS0    = 0x0002U;   /* Start count operation for Unit 0, CH5 */ 

}	/* end of function hwsetup() */

/******************************************************************************
Exported global functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
* Function Name: __low_level_init
* Description  : low level init
* Arguments    : none
* Return Value : none
******************************************************************************/
int __low_level_init(void)
{
	DI();
	systeminit();
	CLOCK_Init();
	hwsetup();
	EI();
	
	return MD_TRUE;

}	/* end of function __low_level_init() */



