
/******************************************************************************
* DISCLAIMER

* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized.

* This software is owned by Renesas Electronics Corporation and is protected under 
* all applicable laws, including copyright laws.

* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES 
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, 
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
* PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY 
* DISCLAIMED.

* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES 
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS 
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

* Renesas reserves the right, without notice, to make changes to this 
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and 
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer

******************************************************************************/
/******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : hwsetup.c
* Version      : 2.0
* Device(s)    : RL78G14
* Tool-Chain   : RL78 IAR Embedded Workbench V1.20.x
* OS           : none
* H/W Platform : YRMCKITRL78G14 V2.0
* Description  : main application - Source build
* Operation    :
* Limitations  :
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0    Production release plus rebuild for new IAR EW release
*                              modified file organisation to support object creation
*******************************************************************************/



/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "globdef.h"
#include "motorcontrol.h"
#include "userif.h"
#include "stl.h"
#include "stl_support.h"

/*******************************************************************************
Pragma Definitions for Linker
*******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/* CRC test range variable structure */ 
static CHECKSUM_CRC_TEST_AREA checksum_crc;


/******************************************************************************
Imported global variables and functions
******************************************************************************/
extern UCHAR resetflag;


/******************************************************************************
* Function Name: main
* Description  : main program
* Arguments    : none
* Return Value : none
******************************************************************************/
void main(void)
{

	/* interrupt disable */
	__disable_interrupt();
        
        
        /* Check RESET source */
	if(resetflag != 0)
        {
          SET_LD3_ON;         /* Error RESET (WDOG, Partity error Illegal Instruction etc) */
          STOP();             /* As error occured STOP further operation */
        }
        else
        {
          SET_LD3_OFF;       /* Normal RESET/Power On */
        }


/*******************************************************************************
	UIF_R.var.flg		bit 0		march C initial test OK
	UIF_R.var.flg		bit 1		all register tests OK
	UIF_R.var.flg		bit 2		hw clock test OK
	UIF_R.var.flg		bit 3		peripheral CRC test OK
	UIF_R.var.flg		bit 4		NU
	UIF_R.var.flg		bit 5		NU
	UIF_R.var.flg		bit 6		NU
	UIF_R.var.flg		bit 7		board alarm active
	UIF_R.var.flg		bit 8		cpu core general registers test failed
	UIF_R.var.flg		bit 9		PSW register test failed
	UIF_R.var.flg		bit A		SP register test failed
	UIF_R.var.flg		bit B		CS register test failed
	UIF_R.var.flg		bit C		ES register test failed
	UIF_R.var.flg		bit D		NU
	UIF_R.var.flg		bit E		NU
	UIF_R.var.flg		bit F		auto demo mode on
*******************************************************************************/
	UIF_R.var.flg = 0x000F;			// reset status flags

#ifndef DEBUG

	/* delay for bus voltage stabilization */
	DELAY100US(1000)		// 100ms

/*******************************************************************************
	SELF TEST BEGINS
*******************************************************************************/

/*******************************************************************************
	stl_RL78_march_c_initial
		performs initial IEC March_c test (called in cstartup)
		the result is in the global variable g_resultBool, which is TRUE
		if the test failed.
*******************************************************************************/	
	if (g_resultBool)
	{
		UIF_R.var.flg &= WCLR0;
	}

/*******************************************************************************
	stl_RL78_cpu_registertest
		performs a functional test of the CPU core register
		( AX, BC, DE, HL - all four banks ). The test is non destructive,
		all registers are saved and restored on completion of the test function.
*******************************************************************************/
	g_resultBool = stl_RL78_registertest();
	if (g_resultBool)
	{
		UIF_R.var.flg &= WCLR1;
		UIF_R.var.flg |= WSET8;
	}

/*******************************************************************************
	stl_RL78_cpu_registertest_psw
		performs a functional test of the PSW,  CPU core register .
		The test is non destructive, The PSW register content is saved and
		restored on the completion of the test function.
*******************************************************************************/
    g_resultBool = stl_RL78_registertest_psw();
	if (g_resultBool)
	{
		UIF_R.var.flg &= WCLR1;
		UIF_R.var.flg |= WSET9;
	}

/*******************************************************************************
	stl_RL78_cpu_registertest_stack
		performs a functional test of the SP, CPU core register .
		The test is non destructive, the stack register content is saved and 
		restored on completion of the test function.
*******************************************************************************/
	g_resultBool = stl_RL78_registertest_stack();
	if (g_resultBool)
	{
		UIF_R.var.flg &= WCLR1;
		UIF_R.var.flg |= WSETA;
	}

/*******************************************************************************
	stl_RL78_cpu_registertest_cs
		Performs a functional test of the CS CPU core register .
		The test is non destructive, the CS register content is saved and 
		restored on completion of the test function.
*******************************************************************************/
	g_resultBool = stl_RL78_registertest_cs();
	if (g_resultBool)
	{
		UIF_R.var.flg &= WCLR1;
		UIF_R.var.flg |= WSETB;
	}

/*******************************************************************************
	stl_RL78_cpu_registertest_es
		Performs a functional test of the ES CPU core register .
		The test is non destructive, the ES register content is saved and 
		restored on completion of the test function.
*******************************************************************************/
	g_resultBool = stl_RL78_registertest_es();
	if (g_resultBool)
	{
		UIF_R.var.flg &= WCLR1;
		UIF_R.var.flg |= WSETC;
	}

/*******************************************************************************
	stl_RL78_hw_clocktest
		Example of a system clock test using the on chip hardware (TAU ch1). 
		The system clock (32MHz) is measured during the period of the internal
		15KHz Low Speed clock. The timer channel of the TAU is configured to 
                measure the LS Oscillator clock period and the capture value is compared 
                against a reference window values, defined in stl_clocktest.h
*******************************************************************************/
	g_resultBool = stl_RL78_hw_clocktest();
	if (g_resultBool)
	{
		UIF_R.var.flg &= WCLR2;
	}

/*******************************************************************************
	stl_RL78_peripheral_crc.asm
		This function calculates the cyclic redundancy check (CRC) of data
		starting at the address in "start_address" over the range set in "length". 
		The starting CRC value is set in "g_crc" The final CRC result is stored in 
		the "CRCD" register is returned for verification where it is compared 
		against a reference value.
	INPUT:	start address (checksum_crc.start_address)
			Test end address (length) (checksum_crc.length )
	RETURN:	CRC result over the specified Flash area
*******************************************************************************/

    g_referenceCalc_Done  = 0U;	/* Set reference CRC calculation not done */

    IAWCTL  |= 0x80U;                             /* Illegal access detection setting */
                                                  /* Bit 7 = 0 Detection Disabled */ 
                                                  /* Bit 7 = 1 Detection enabled */

    g_crc   = 0x0000U;                            /* Clear CRC result variable */
    checksum_crc.start_address = 0x00001000U;     /* CRC start address */
    checksum_crc.length        = 0x0000F000U;     /* Set CRC end address (Length of calculation - 1) */

    /* Execute peripheral CRC calculation */
    g_crcASM = stl_RL78_peripheral_crc(g_crc, &checksum_crc);

    /* calculate reference peripheral CRC  !! Do this only once !! */
    if (g_referenceCalc_Done == 0U)                /* Do if reference CRC not already calculated */
    {      
        g_crc = 0x0U;
        g_pStAdr =  (uint8_t __far *)checksum_crc.start_address;
        g_crc_length = checksum_crc.length;
        g_reference_crc  = reference_crc_calculation(g_crc, g_crc_length, g_pStAdr);
    }
    
    /* Check CRC result against calculated reference */    
    if (g_crcASM  != g_reference_crc)
    {
		UIF_R.var.flg &= WCLR3;
    }

/*******************************************************************************
	SELF TEST ENDED
*******************************************************************************/


        /* Turn Parity error detection on */
        RPECTL |= 0x80;
    

#endif	/* If not DEBUG */

#ifdef DELAY_TUNING
	while (1)	/* delay tuning */
	{
		SET_LD3_ON
		DELAY100US(10)
		SET_LD3_OFF
		DELAY100US(10)
	}
#endif	/* DELAY_TUNING */

	/* turn on the transistor which cuts off the inrush current limitation */
	SET_USB_PWR_ON

	/* parameters table upload */
	if (ini_eqp() != 0)
	{
		UIF_R.var.all = EQP_ALL;
		UIF_R.var.flg |= WSET7;
	}

	/* parameters init */
	MC_IniPar();
	MC_ReadButtons();           // Set defalt button settings


	/* A/D init and current offsets reading */
	MC_SetOff();

	/* communication init */
	ini_232_pc();

	/* pwm init and start */
#ifdef DEBUG_PWM
        
	MC_ini_deb_pwm();
        
#endif	/* DEBUG_PWM */
        
	MC_IniPWM();
	MC_StartPWM();

	/* enable interrupt */
	__enable_interrupt();

	/* main loop */
	while (1)
	{

		/* time synchronization */
		while (MC_WaitSync())
		{
			/* this time can be used for asynchronous functions */
		}

		/* timing debug */
		M_LOOP_TIM_DEB_LD3_ON

		/* control functions */
		MC_ConFil();
		MC_SpeedRamp();

		/* serial communication management */
		ges_232_pc();

		/* general management functions */
		ges_led();
		MC_ReadButtons();

		/* eeprom parameters update */
		MC_UpdPar();
		if (man_eqp() != 0)
		{
			UIF_R.var.all = EQP_ALL;
			UIF_R.var.flg |= WSET7;
		}

		/* timing debug */
		M_LOOP_TIM_DEB_LD3_OFF

	}	/* end of main loop */
} /* end of main function */

