/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
*******************************************************************************/
/* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.   */
/*******************************************************************************
* File Name : mcrplib.h
* Version : 2.0
* Device(s) : RL78/G14
* Tool-Chain : RL78 IAR Embedded Workbench V1.20.x
* OS : None
* H/W Platform : YRMCKITRL78G14 Rev 2.0
* Description  : Motor control library functions prototypes
* Operation :
* Limitations : None
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0    Production release plus rebuild for new IAR EW release
*                              modified file organisation to support object creation
******************************************************************************/


/* THIS FILE IS INCLUDED ALSO IN LIBRARY SO NOTHING CAN BE MODIFIED! */

#ifndef _MCRPLIB_H
#define _MCRPLIB_H


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "multiply.h"
#include "typedefine.h"


/******************************************************************************
Macro definitions
******************************************************************************/

/* phase and speed estimation */
#define FLUX_LP_CONST	  ( 1024 )
#define FLUX_LP_SHFT	  ( 10 )
//#define SPEED_NSAMPLES  ( 16 )
//#define SH_NSAMPLES	  ( 4 )
//#define SPEED_NSAMPLES  ( 8 )
//#define SH_NSAMPLES	  ( 3 )
#define SPEED_NSAMPLES	  ( 4 )
#define SH_NSAMPLES	  ( 2 )
#define SPEED_LP_CONST	  ( 16 )
#define SPEED_LP_SHFT	  ( 4 )

/* numerical constants */
#ifndef S16_CONST
#define S16_CONST
#define	AMP_S16_CONST	  ( 16384 )	// amplification
#define	SH_S16_CONST	  ( 14 )	// shifts
#define	ONETHIRD	  ( 5461 )	// 2^14*(1/3)=5461.33333
#define	TWOTHIRDS	  ( 10923 )	// 2^14*(2/3)=10922.66666
#define	SQRT3		  ( 28378 )	// 2^14*SQUAREROOT(3)=28377.92043
#define	ONEDIVSQRT3	  ( 9459 )	// 2^14/SQUAREROOT(3)=9459.30681
#define	SQRT2		  ( 23170 )	// 2^14*SQUAREROOT(2)=23170.47501
#define	ONEDIVSQRT2	  ( 11585 )	// 2^14/SQUAREROOT(2)=11585.2375
#define SQRT3DIVSQRT2	  ( 20066 )	// 2^14*SQUAREROOT(3/2)=20066.21997
#define SQRT2DIVSQRT3	  ( 13377 )	// 2^14*SQUAREROOT(2/3)=13377.47998
#endif	// S16_CONST

/* trigonometric function related definitions */
#ifndef ANGLES
#define ANGLES

/* normalizations */
#define NORMSIN		( 16384 )	// amplification
#define SHFTSIN		( 14 )	        // shifts

/* angles definitions */
#define	PISIXTHS	( 5461 )	// 0x1555 030
#define	PIFOURTHS	( 8192 )	// 0x2000 045
#define	PITHIRDS	( 10923 )	// 0x2AAB 060
#define	PIHALVES	( 16384 )	// 0x4000 090
#define	TWOPITHIRDS	( 21845 )	// 0x5555 120
#define	THREEPIFOURTHS	( 24576 )	// 0x6000 135
#define	FIVEPISIXTHS	( 27307 )	// 0x6AAB 150
#define	PI		( 32768 )	// 0x8000 180
#define	SEVENPISIXTHS	( 38229 )	// 0x9555 210
#define	FIVEPIFOURTHS	( 40960 )	// 0xA000 225
#define	FOURPITHIRDS	( 43691 )	// 0xAAAB 240
#define	THREEPIHALVES	( 49152 )	// 0xC000 270
#define	FIVEPITHIRDS	( 54613 )	// 0xD555 300
#define	SEVENPIFOURTHS	( 57344 )	// 0xE000 315
#define	ELEVENPISIXTHS	( 60075 )	// 0xEAAB 330
#define	TWOPI		( 65536 )	// 0x00010000 LONG; IF WORD ACCESS GIVES 0
#define	ANGMOD		( 0xFFFF )	// TWOPI-1
#define	ANG1Q		( 0x3FFF )	// select first_quarter_value
#define	QUAMOD		( 0xC000 )	// select quarter
#define	ISCOS		( 0x4000 )	// table(first_quarter_value) gives cos
#define	ISNEG		( 0x8000 )	// sin is neg

#define TWOPIF		( 6.283185307 )
#define KAF		(TWOPI / TWOPIF)    // angles conversion constant
#define KA		((uint16_t)KAF)	    // angles conversion constant

#endif	/* ANGLES */

/******************************************************************************
Functions Prototypes
******************************************************************************/

/* TRIGONOMETRIC FUNCTIONS ***************************************************/

/******************************************************************************
* Function Name: McrpLib_sin
* Description  : y=NORMSIN*sin(ang)
* Arguments    : ang=KA*angle[rad], 0<=ang<TWOPI (0<=angle<2pi)
* Return Value : normalized sin value y, |y|<=NORMSIN
******************************************************************************/
int16_t McrpLib_sin(int16_t ang);

/******************************************************************************
* Function Name: McrpLib_cos
* Description  : y=NORMSIN*cos(ang)
* Arguments    : ang=KA*angle[rad], 0<=ang<TWOPI (0<=angle<2pi)
* Return Value : normalized cos value y, |y|<=NORMSIN
******************************************************************************/
int16_t McrpLib_cos(int16_t ang);

/******************************************************************************
* Function Name: McrpLib_sinarcos
* Description  : y=NORMSIN*sin(arcos(x/NORMSIN))
* Arguments    : normalized cos(angle) value x, 0<=x<=NORMSIN, (0<=angle<=pi/2)
* Return Value : normalized sin(angle) value y, 0<=y<=NORMSIN
******************************************************************************/
uint16_t McrpLib_sinarcos(uint16_t x);

/******************************************************************************
* Function Name: McrpLib_atan2
* Description  : ang=KA*arctan(y/x)
* Arguments    : amplified value x=A*cos(angle),
*                amplified value y=A*sin(angle),
*                0<A<2^15, (0<=angle<=2pi)
* Return Value : internal representation value of angle: ang=KA*angle
******************************************************************************/
int16_t McrpLib_atan2(int16_t x, int16_t y);

/* END OF TRIGONOMETRIC FUNCTIONS ********************************************/

/* TRANSFORMATION FUNCTIONS **************************************************/

/******************************************************************************
* Function Name: McrpLib_angle_set
* Description  : internal angle variable setting
* Arguments    : ang=KA*angle[rad], 0<=ang<TWOPI (0<=angle<2pi)
* Return Value : none
* Modifies     : internal_angle (base for Park transformations)
******************************************************************************/
void McrpLib_angle_set(uint16_t ang);

/******************************************************************************
* Function Name: McrpLib_uvw_alphabeta
* Description  : unitary gain transformation (u, v, w)->(alpha, beta):
*                alpha=u, beta=(u+2*v)/sqrt(3)
* Arguments    : u and v components (both <16384),
*                alpha component address, beta component address
* Return Value : none
* Modifies     : alpha, beta components
* Note         : result not guaranteed if vector amplitude greater than 16383
******************************************************************************/
void McrpLib_uvw_alphabeta( int16_t u, int16_t v, int16_t *alpha, int16_t *beta);

/******************************************************************************
* Function Name: McrpLib_alphabeta_uvw
* Description  : unitary gain transformation (alpha, beta)->(u, v, w):
*                u=alpha, v=((sqrt(3)*alpha-beta)/2
* Arguments    : alpha and beta components (both <16384),
*                u component address, v component address
* Return Value : none
* Modifies     : u, v components
* Note         : result not guaranteed if vector amplitude greater than 16383
******************************************************************************/
void McrpLib_alphabeta_uvw( int16_t alpha, int16_t beta, int16_t *u, int16_t *v);

/******************************************************************************
* Function Name: McrpLib_alphabeta_dq
* Description  : unitary gain transformation (alpha, beta)->(d, q):
*                d=alpha*cos(internal_angle)+beta*sin(internal_angle)
*                q=alpha*cos(internal_angle)-beta*sin(internal_angle)
* Arguments    : alpha component, beta component,
*                d component address, q component address
* Return Value : none
* Modifies     : d, q components
******************************************************************************/
void McrpLib_alphabeta_dq( int16_t alpha, int16_t beta, int16_t *d, int16_t *q);

/******************************************************************************
* Function Name: McrpLib_dq_alphabeta
* Description  : unitary gain transformation (d, q)->(alpha, beta):
*                alpha=d*cos(internal_angle)-q*sin(internal_angle)
*                beta=d*sin(internal_angle)+q*cos(internal_angle)
* Arguments    : d component, q component,
*                alpha component address, beta component address
* Return Value : none
* Modifies     : alpha, beta components
******************************************************************************/
void McrpLib_dq_alphabeta( int16_t d, int16_t q, int16_t *alpha, int16_t *beta);

/* END OF TRANSFORMATION FUNCTIONS *******************************************/

/* PMSM DRIVE FUNCTIONS ******************************************************/

/******************************************************************************
* Function Name: McrpLib_FluxEst
* Description  : magnetic flux estimation filter
* Arguments    : bemf voltage component x[volt/64], filter memories vector mem[]
* Return Value : flux component f[weber/1024]
******************************************************************************/
uint16_t McrpLib_FluxEst(int16_t x, int32_t *mem);

/******************************************************************************
* Function Name: McrpLib_SpeedRes
* Description  : speed estimation algorithm memories reset
* Arguments    : actual phase (internal representation)
* Return Value : none
******************************************************************************/
void McrpLib_SpeedRes(uint16_t phase);

/******************************************************************************
* Function Name: McrpLib_SpeedEst
* Description  : speed estimation: speed is expressed as phase difference
*                between samples far s_ns each other (s(n)=ph(n)-ph(n-s_ns))
* Arguments    : actual phase (internal representation)
* Return Value : estimated speed (internal representation)
* Note         : max speed allowed is lesser than: 0.5*Fs/ns
*                es. if Fs=8kHz and ns=4, then max freq < 1kHz
******************************************************************************/
int16_t McrpLib_SpeedEst(uint16_t phase);

/* END OF PMSM DRIVE FUNCTIONS ************************************************/


#endif /* _MCRPLIB_H */

