/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
*******************************************************************************/
/* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.   */
/*******************************************************************************
* File Name : motorcontrol.h
* Version : 2.0
* Device(s) : RL78/G14
* Tool-Chain : RL78 IAR Embedded Workbench V1.20.x
* OS : None
* H/W Platform : YRMCKITRL78G14 Rev 2.0
* Description  : header for motorcontrol.c
* Operation :
* Limitations : None
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0    Production release plus rebuild for new IAR EW release
*                              modified file organisation to support object creation
******************************************************************************/


#ifndef _MOTORCONTROL_H
#define _MOTORCONTROL_H

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "typedefine.h"

/******************************************************************************
Type definitions
******************************************************************************/
typedef enum {
	END_DEMO,
	RAMP_UP_0,
	VEL_NEG,
	RAMP_DOWN,
	VEL_POS,
	RAMP_UP,
} ad_state_t;


/******************************************************************************
Functions Prototypes
******************************************************************************/

#ifdef DEBUG_PWM
/******************************************************************************
* Function Name: MC_ini_deb_pwm
* Description  : pwm output init
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_ini_deb_pwm(void);

/******************************************************************************
* Function Name: MC_set_deb_pwm_duty
* Description  : pwm output duty cycle setting
* Arguments    : duty cycle (0 -> 0%, 65535 -> 100%)
* Return Value : none
******************************************************************************/
void MC_set_deb_pwm_duty(uint16_t duty);

#endif	/* DEBUG_PWM */

/******************************************************************************
* Function Name: MC_WaitSync
* Description  : time synchronization control function
* Arguments    : none
* Return Value : 0 if main loop time is elapsed, 1 if not
******************************************************************************/
unsigned char MC_WaitSync(void);

/******************************************************************************
* Function Name: MC_SetOff
* Description  : current offsets measurement
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_SetOff(void);

/******************************************************************************
* Function Name: MC_IniPar
* Description  : internal parameters initialization, using eeprom values
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_IniPar(void);

/******************************************************************************
* Function Name: MC_UpdPar
* Description  : internal parameters update, if eeprom values are changed
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_UpdPar(void);


/******************************************************************************
* Function Name: MC_ConFil
* Description  : management of measured values, to give them to the GUI
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_ConFil(void);

/******************************************************************************
* Function Name: MC_StartPWM
* Description  : start pwm and main interrupt (but not the outputs)
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_StartPWM(void);


/******************************************************************************
* Function Name: MC_SpeedRamp
* Description  : speed increasing and decreasing ramps
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_SpeedRamp(void);

#endif /* _MOTORCONTROL_H */

