/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
*******************************************************************************/
/* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.   */
/*******************************************************************************
* File Name : multiply.h
* Version : 2.0
* Device(s) : RL78/G14
* Tool-Chain : RL78 IAR Embedded Workbench V1.20.x
* OS : None
* H/W Platform : YRMCKITRL78G14 Rev 2.0
* Description  : header interface for multiply.s87
* Operation :
* Limitations : None
********************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0    Production release plus rebuild for new IAR EW release
*                              modified file organisation to support object creation
*******************************************************************************/

#ifndef _MULTIPLY_H
#define _MULTIPLY_H

/******************************************************************************
Functions Prototypes
******************************************************************************/

/******************************************************************************
* Function Name: mulu, mul
* Description  : 16*16=32 unsigned or signed multiply
* Arguments    : the two (unsigned or signed) short arguments
* Return Value : the (unsigned or signed) long result
******************************************************************************/
extern unsigned long mulu(unsigned short, unsigned short);
extern signed long mul(signed short, signed short);


/******************************************************************************
* Function Name: sharX, 1<=X<=16
* Description  : 32>>X=16 arithmetical shift down
* Arguments    : the signed long argument of the shift
* Return Value : the signed short result
* NOTE         : only the lowest 16bits of the shift result are given!
******************************************************************************/
extern signed short shar1(signed long);
extern signed short shar2(signed long);
extern signed short shar3(signed long);
extern signed short shar4(signed long);
extern signed short shar5(signed long);
extern signed short shar6(signed long);
extern signed short shar7(signed long);
extern signed short shar8(signed long);
extern signed short shar9(signed long);
extern signed short shar10(signed long);
extern signed short shar11(signed long);
extern signed short shar12(signed long);
extern signed short shar13(signed long);
extern signed short shar14(signed long);
extern signed short shar15(signed long);
extern signed short shar16(signed long);


/******************************************************************************
* Function Name: sharX_l, 1<=X<=16
* Description  : 32>>X=32 arithmetical shift down
* Arguments    : the signed long argument of the shift
* Return Value : the signed long result
******************************************************************************/
extern signed long shar1_l(signed long);
extern signed long shar2_l(signed long);
extern signed long shar3_l(signed long);
extern signed long shar4_l(signed long);
extern signed long shar5_l(signed long);
extern signed long shar6_l(signed long);
extern signed long shar7_l(signed long);
extern signed long shar8_l(signed long);
extern signed long shar9_l(signed long);
extern signed long shar10_l(signed long);
extern signed long shar11_l(signed long);
extern signed long shar12_l(signed long);
extern signed long shar13_l(signed long);
extern signed long shar14_l(signed long);
extern signed long shar15_l(signed long);
extern signed long shar16_l(signed long);


/******************************************************************************
* Function Name: mulusharX, mulsharX, 1<=X<=16
* Description  : ((16*16)>>K=16 multiply and arithmetical shift down
* Arguments    : the two (unsigned or signed) short arguments
* Return Value : the (unsigned or signed) short result
* NOTE         : the result has no sense if the operation result doesn't fit
*                in 16 bits!
******************************************************************************/
extern unsigned short mulushar1(unsigned short, unsigned short);
extern unsigned short mulushar2(unsigned short, unsigned short);
extern unsigned short mulushar3(unsigned short, unsigned short);
extern unsigned short mulushar4(unsigned short, unsigned short);
extern unsigned short mulushar5(unsigned short, unsigned short);
extern unsigned short mulushar6(unsigned short, unsigned short);
extern unsigned short mulushar7(unsigned short, unsigned short);
extern unsigned short mulushar8(unsigned short, unsigned short);
extern unsigned short mulushar9(unsigned short, unsigned short);
extern unsigned short mulushar10(unsigned short, unsigned short);
extern unsigned short mulushar11(unsigned short, unsigned short);
extern unsigned short mulushar12(unsigned short, unsigned short);
extern unsigned short mulushar13(unsigned short, unsigned short);
extern unsigned short mulushar14(unsigned short, unsigned short);
extern unsigned short mulushar15(unsigned short, unsigned short);
extern unsigned short mulushar16(unsigned short, unsigned short);
extern signed short mulshar1(signed short, signed short);
extern signed short mulshar2(signed short, signed short);
extern signed short mulshar3(signed short, signed short);
extern signed short mulshar4(signed short, signed short);
extern signed short mulshar5(signed short, signed short);
extern signed short mulshar6(signed short, signed short);
extern signed short mulshar7(signed short, signed short);
extern signed short mulshar8(signed short, signed short);
extern signed short mulshar9(signed short, signed short);
extern signed short mulshar10(signed short, signed short);
extern signed short mulshar11(signed short, signed short);
extern signed short mulshar12(signed short, signed short);
extern signed short mulshar13(signed short, signed short);
extern signed short mulshar14(signed short, signed short);
extern signed short mulshar15(signed short, signed short);
extern signed short mulshar16(signed short, signed short);


/******************************************************************************
* Function Name: divu
* Description  : 32:16=16 unsigned division
* Arguments    : the unsigned long dividend and the unsigned short divisor
* Return Value : the unsigned short result
* NOTE         : the result has no sense if the operation result doesn't fit
*                in 16 bits!
******************************************************************************/
extern unsigned short divu(unsigned long, unsigned short);


/******************************************************************************
* Function Name: div
* Description  : 32:16=16 signed division
* Arguments    : the signed long dividend and the signed short divisor
* Return Value : the signed short result
* NOTE         : the result has no sense if the operation result doesn't fit
*                in 16 bits!
******************************************************************************/
extern signed short div(signed long, signed short);


/******************************************************************************
* Function Name: muldivu
* Description  : 16*16/16=16 unsigned multiply and division
* Arguments    : the two unsigned short arguments of the multiply, and the
*                unsigned short divisor
* Return Value : the unsigned short result
* NOTE         : the result has no sense if the operation result doesn't fit
*                in 16 bits!
******************************************************************************/
extern unsigned short muldivu(unsigned short, unsigned short, unsigned short);


/******************************************************************************
* Function Name: muldiv
* Description  : 16*16/16=16 signed multiply and division
* Arguments    : the two signed short arguments of the multiply, and the
*                signed short divisor
* Return Value : the signed short result
* NOTE         : the result has no sense if the operation result doesn't fit
*                in 16 bits!
******************************************************************************/
extern signed short muldiv(signed short, signed short, signed short);


#endif /* _MULTIPLY_H */

