
/******************************************************************************
* DISCLAIMER

* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized.

* This software is owned by Renesas Electronics Corporation and is protected under 
* all applicable laws, including copyright laws.

* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES 
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, 
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
* PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY 
* DISCLAIMED.

* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES 
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS 
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

* Renesas reserves the right, without notice, to make changes to this 
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and 
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer

******************************************************************************/

/******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : par_tab.c
* Version      : 2.0
* Device(s)    : RL78G14
* Tool-Chain   : RL78 IAR Embedded Workbench V1.20.x
* H/W Platform : YRMCKITRL78G14 V2.0
* Description  : Parameter table management - Source build
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0   First Release (Prototype)
*         : 01.06.2012 V1.1   Updated prototype release to include motor control library
                              plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0   Production release plus rebuild for new IAR EW release
*                             modified file organisation to support object creation
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "globdef.h"

#ifdef DATA_FLASH_LINKED
#include "fdl_types.h"
#include "fdl.h"
#include "fdl_descriptor.h"
#endif	/* DATA_FLASH_LINKED */


/******************************************************************************
Macro definitions
******************************************************************************/

#ifdef DATA_FLASH_LINKED

/* macros related to data flash */
#define NUM_EQP		  ( (uint16_t) ((NUM_USED + 1) / 2) )
#define EQPBUSY		  ( FAL_BUSY == df_request.status_enu )
#define	WAITBUSY	  { while ( EQPBUSY ) FAL_Handler(); }
#define	READ( ADD )	  { df_request.index_u16 = ADD;	\ df_request.command_enu = FAL_CMD_READ_WORD; \ FAL_Execute(&df_request); }
#define READNOK		  ( df_request.status_enu != FAL_OK )
#define READVAL		  ( df_request.data_u32 )

#define	WRITE( ADD, VAL ) { df_request.index_u16 = ADD;	\ df_request.command_enu = FAL_CMD_WRITE_WORD;	\ df_request.data_u32 = VAL; \
			    FAL_Execute(&df_request); }

#define WRITENOK  	  ( df_request.status_enu != FAL_OK )

#define	BLANKCHK( ADD )	  { df_request.index_u16 = ADD;	\ df_request.command_enu = FAL_CMD_BLANKCHECK_WORD; \
			    FAL_Execute(&df_request); }

#define NOTBLANK	  ( FAL_ERR_BLANKCHECK == df_request.status_enu )
#define BLANK		  ( FAL_OK == df_request.status_enu )
#define	ERASE( ADD )	  { df_request.index_u16 = ADD;	\ df_request.command_enu = FAL_CMD_ERASE_BLOCK;	\
			    FAL_Execute(&df_request); }

#define ERASENOK          ( df_request.status_enu != FAL_OK )

/* end of macros related to data flash */
#else	/* DATA_FLASH_LINKED */

#ifdef EXT_EEPROM
/* macros related to external eeprom */

#define NUM_EQP		( (uint16_t) (NUM_USED) )
#define MAX_EQP		( NUM_EQP-1 )

#define MAX_WAIT	( 50 )	                // 50 ms
#define	NL_W_EQP	( 5 )	                // 5*10ms=50ms max

#define IOC_OUT		( 0 )
#define IOC_IN		( 1 )
#define	SCLIOC		( PM6_bit.no0 )		// P60/SCLA0    UP_17 OUT_EEPSCL
#define	EEPSCL		( P6_bit.no0 )		// P60/SCLA0    UP_17 OUT_EEPSCL
#define	SDAIOC		( PM6_bit.no1 )		// P61/SDAA0    UP_18 IO_EEPSDA
#define	EEPSDA		( P6_bit.no1 )		// P61/SDAA0    UP_18 IO_EEPSDA
#define	EEPSDAIN	( P6_bit.no1 )		// P61/SDAA0    UP_18 IO_EEPSDA

#define PUT_SCL_OUT	{ SCLIOC=IOC_OUT; }
#define PUT_SCL_IN	{ SCLIOC=IOC_IN; }
#define PUT_SDA_OUT	{ SDAIOC=IOC_OUT; }
#define PUT_SDA_IN	{ SDAIOC=IOC_IN; }
#define	SET_SCL		{ EEPSCL=1; }
#define	CLR_SCL		{ EEPSCL=0; }
#define SET_SDA		{ PUT_SDA_IN }
#define CLR_SDA		{ PUT_SDA_OUT EEPSDA=0; }
#define	SDA_IN_H	(EEPSDAIN!=0)
#define	SDA_IN_L	(EEPSDAIN==0)

#define	WAIT_SCL	DELAY1US(2)		// clock half period: minimum 1.25 microsec

#define	CODE_ADD_W	( 0xA0U )		// 1010=CODE 000=ADD 0=WRITE
#define	CODE_ADD_R	( 0xA1U )		// 1010=CODE 000=ADD 1=READ

//#define	ADD2BYTES	// if only 1 byte addressing please comment this

/* end of macros related to external eeprom */
#endif	/* EXT_EEPROM */

#endif	/* DATA_FLASH_LINKED */

#define CHKVAL_W	    ( 0xA5A5 )		// not blank
#define CHKVAL		    ( 0xA5A5A5A5 )	// not blank

#define	EQP_ERROR	    ( eqp_flg.bit.b0 == 1 )
#define	EQP_ERASING	    ( eqp_flg.bit.b1 == 1 )
#define	EQP_UPDATED	    ( eqp_flg.bit.b2 == 1 )
#define	EQP_MODREQ	    ( eqp_flg.bit.b3 == 1 )
#define	LIBERA_4	    ( eqp_flg.bit.b4 == 1 )
#define	LIBERA_5	    ( eqp_flg.bit.b5 == 1 )
#define	LIBERA_6	    ( eqp_flg.bit.b6 == 1 )
#define	LIBERA_7	    ( eqp_flg.bit.b7 == 1 )

#define	NOT_EQP_ERROR	    ( eqp_flg.bit.b0 == 0 )
#define	NOT_EQP_ERASING	    ( eqp_flg.bit.b1 == 0 )
#define	NOT_EQP_UPDATED	    ( eqp_flg.bit.b2 == 0 )
#define	NOT_EQP_MODREQ	    ( eqp_flg.bit.b3 == 0 )
#define	NOT_LIBERA_4	    ( eqp_flg.bit.b4 == 0 )
#define	NOT_LIBERA_5	    ( eqp_flg.bit.b5 == 0 )
#define	NOT_LIBERA_6	    ( eqp_flg.bit.b6 == 0 )
#define	NOT_LIBERA_7	    ( eqp_flg.bit.b7 == 0 )

#define	SET_EQP_ERROR	    { eqp_flg.bit.b0 = 1; }
#define	SET_EQP_ERASING	    { eqp_flg.bit.b1 = 1; }
#define	SET_EQP_UPDATED	    { eqp_flg.bit.b2 = 1; }
#define	SET_EQP_MODREQ	    { eqp_flg.bit.b3 = 1; }
#define	SET_LIBERA_4	    { eqp_flg.bit.b4 = 1; }
#define	SET_LIBERA_5	    { eqp_flg.bit.b5 = 1; }
#define	SET_LIBERA_6	    { eqp_flg.bit.b6 = 1; }
#define	SET_LIBERA_7	    { eqp_flg.bit.b7 = 1; }

#define	RES_EQP_ERROR	    { eqp_flg.bit.b0 = 0; }
#define	RES_EQP_ERASING	    { eqp_flg.bit.b1 = 0; }
#define	RES_EQP_UPDATED	    { eqp_flg.bit.b2 = 0; }
#define	RES_EQP_MODREQ	    { eqp_flg.bit.b3 = 0; }
#define	RES_LIBERA_4	    { eqp_flg.bit.b4 = 0; }
#define	RES_LIBERA_5	    { eqp_flg.bit.b5 = 0; }
#define	RES_LIBERA_6	    { eqp_flg.bit.b6 = 0; }
#define	RES_LIBERA_7	    { eqp_flg.bit.b7 = 0; }


/******************************************************************************
* Typedef definitions
******************************************************************************/

#ifdef DATA_FLASH_LINKED
typedef struct
{
	int16_t	higher;
	int16_t	lower;
} twowords_t;

typedef union
{
	uint32_t  longval;
	twowords_t  wordval;
} df_val_t;
#endif	/* DATA_FLASH_LINKED */


/******************************************************************************
Const definitions
******************************************************************************/

const uint16_t
	par_min[NUM_PAR]={	/* MINIMUM VALUES */
		0,		//  0 SEL_OP
		200,		//  1 RPM_MIN
		1000,		//  2 RPM_MAX
		1,		//  3 R_ACC
		1,		//  4 R_DEC
		1,		//  5 C_POLI
		0,		//  6 I_START
		0,		//  7 I_MAX
		0,		//  8 R_STA
		0,		//  9 L_SYN
		300,		// 10 STP_TIM
		0,		// 11 KP_CUR
		0,		// 12 KI_CUR
		0,		// 13 KP_VEL
		0,		// 14 KI_VEL
		0,		// 15 PAR_15
		0,		// 16 PAR_16
		0,		// 17 PAR_17
		0,		// 18 PAR_18
		0,		// 19 PAR_19
		0,		// 20 PAR_20
		0,		// 21 PAR_21
		0,		// 22 PAR_22
		0,		// 23 PAR_23
		0,		// 24 PAR_24
		0,		// 25 PAR_25
		0,		// 26 PAR_26
		0,		// 27 PAR_27
		0,		// 28 PAR_28
		0,		// 29 PAR_29
		0,		// 30 PAR_30
		0,		// 31 PAR_31
		0,		// 32 PAR_32
		0,		// 33 PAR_33
		0,		// 34 PAR_34
		0,		// 35 PAR_35
		0,		// 36 PAR_36
		0,		// 37 PAR_37
		0,		// 38 PAR_38
		0,		// 39 PAR_39
		0,		// 40 PAR_40
		0,		// 41 PAR_41
		0,		// 42 PAR_42
		0,		// 43 PAR_43
		0,		// 44 PAR_44
		0,		// 45 PAR_45
		0,		// 46 PAR_46
		0,		// 47 PAR_47
		0,		// 48 PAR_48
		0,		// 49 PAR_49
		0,		// 50 PAR_50
		0,		// 51 PAR_51
		0,		// 52 PAR_52
		0,		// 53 PAR_53
		0,		// 54 PAR_54
		0,		// 55 PAR_55
		0,		// 56 PAR_56
		0,		// 57 PAR_57
		0,		// 58 PAR_58
		0,		// 59 PAR_59
		0,		// 60 PAR_60
		0,		// 61 PAR_61
		CHKVAL_W,	// 62 PAR_62
		CHKVAL_W},	// 63 PAR_63
                
      par_def[NUM_PAR]={	/* DEFAULT VALUES */
		0,		//  0 SEL_OP
		RPM_MIN_CUSTOM,	//  1 RPM_MIN
		RPM_MAX_CUSTOM,	//  2 RPM_MAX
		R_ACC_CUSTOM,	//  3 R_ACC
		R_DEC_CUSTOM,	//  4 R_DEC
		C_POLI_CUSTOM,	//  5 C_POLI
		I_START_CUSTOM,	//  6 I_START
		I_MAX_CUSTOM,	//  7 I_MAX
		R_STA_CUSTOM,	//  8 R_STA
		L_SYN_CUSTOM,	//  9 L_SYN
		STP_TIM_CUSTOM,	// 10 STP_TIM
		KP_CUR_CUSTOM,	// 11 KP_CUR
		KI_CUR_CUSTOM,	// 12 KI_CUR
		KP_VEL_CUSTOM,	// 13 KP_VEL
		KI_VEL_CUSTOM,	// 14 KI_VEL
		0,		// 15 PAR_15
		0,		// 16 PAR_16
		0,		// 17 PAR_17
		0,		// 18 PAR_18
		0,		// 19 PAR_19
		0,		// 20 PAR_20
		0,		// 21 PAR_21
		0,		// 22 PAR_22
		0,		// 23 PAR_23
		0,		// 24 PAR_24
		0,		// 25 PAR_25
		0,		// 26 PAR_26
		0,		// 27 PAR_27
		0,		// 28 PAR_28
		0,		// 29 PAR_29
		0,		// 30 PAR_30
		0,		// 31 PAR_31
		0,		// 32 PAR_32
		0,		// 33 PAR_33
		0,		// 34 PAR_34
		0,		// 35 PAR_35
		0,		// 36 PAR_36
		0,		// 37 PAR_37
		0,		// 38 PAR_38
		0,		// 39 PAR_39
		0,		// 40 PAR_40
		0,		// 41 PAR_41
		0,		// 42 PAR_42
		0,		// 43 PAR_43
		0,		// 44 PAR_44
		0,		// 45 PAR_45
		0,		// 46 PAR_46
		0,		// 47 PAR_47
		0,		// 48 PAR_48
		0,		// 49 PAR_49
		0,		// 50 PAR_50
		0,		// 51 PAR_51
		0,		// 52 PAR_52
		0,		// 53 PAR_53
		0,		// 54 PAR_54
		0,		// 55 PAR_55
		0,		// 56 PAR_56
		0,		// 57 PAR_57
		0,		// 58 PAR_58
		0,		// 59 PAR_59
		0,		// 60 PAR_60
		0,		// 61 PAR_61
		CHKVAL_W,	// 62 PAR_62
		CHKVAL_W},	// 63 PAR_63
                
	par_max[NUM_PAR]={	/* MAXIMUM VALUES */
		32767,		//  0 SEL_OP
		5000,		//  1 RPM_MIN
		20000,		//  2 RPM_MAX
		10000,		//  3 R_ACC
		10000,		//  4 R_DEC
		4,		//  5 C_POLI
		5000,		//  6 I_START
		5000,		//  7 I_MAX
		5000,		//  8 R_STA
		5000,		//  9 L_SYN
		10000,		// 10 STP_TIM
		2047,		// 11 KP_CUR
		1023,		// 12 KI_CUR
		4095,		// 13 KP_VEL
		4095,		// 14 KI_VEL
		65535,		// 15 PAR_15
		32767,		// 16 PAR_16
		32767,		// 17 PAR_17
		32767,		// 18 PAR_18
		32767,		// 19 PAR_19
		32767,		// 20 PAR_20
		32767,		// 21 PAR_21
		32767,		// 22 PAR_22
		32767,		// 23 PAR_23
		32767,		// 24 PAR_24
		32767,		// 25 PAR_25
		32767,		// 26 PAR_26
		32767,		// 27 PAR_27
		32767,		// 28 PAR_28
		32767,		// 29 PAR_29
		32767,		// 30 PAR_30
		32767,		// 31 PAR_31
		32767,		// 32 PAR_32
		32767,		// 33 PAR_33
		32767,		// 34 PAR_34
		32767,		// 35 PAR_35
		32767,		// 36 PAR_36
		32767,		// 37 PAR_37
		32767,		// 38 PAR_38
		32767,		// 39 PAR_39
		32767,		// 40 PAR_40
		32767,		// 41 PAR_41
		32767,		// 42 PAR_42
		32767,		// 43 PAR_43
		32767,		// 44 PAR_44
		32767,		// 45 PAR_45
		32767,		// 46 PAR_46
		32767,		// 47 PAR_47
		32767,		// 48 PAR_48
		32767,		// 49 PAR_49
		32767,		// 50 PAR_50
		32767,		// 51 PAR_51
		32767,		// 52 PAR_52
		32767,		// 53 PAR_53
		32767,		// 54 PAR_54
		32767,		// 55 PAR_55
		32767,		// 56 PAR_56
		32767,		// 57 PAR_57
		32767,		// 58 PAR_58
		32767,		// 59 PAR_59
		32767,		// 60 PAR_60
		32767,		// 61 PAR_61
		CHKVAL_W,	// 62 PAR_62
		CHKVAL_W};	// 63 PAR_63


/******************************************************************************
Exported global variables
******************************************************************************/
uint16_t
	par_vec[NUM_PAR];	// parameters vector


/******************************************************************************
Private global variables
******************************************************************************/

#ifdef DATA_FLASH_LINKED

static uint16_t
	df_read_add,
	df_write_add;
static uint16_t
	par_add,
	eqp_add,
	blk_add;
static fal_request_t
	df_request;

#else	/* End if DATA_FLASH_LINKED */

#ifdef EXT_EEPROM

static uint16_t
	sla_ind = 0,
	mas_ind = 0,
	eqp_bsy = 0,
	eqp_aus = 0;

static uint16_t
	eqp_fifo[NUM_PAR],	// fifo valori eqp da aggiornare tramite seriale
	eqp_add[NUM_PAR];	// fifo indirizzi eqp da aggiornare tramite seriale

static int16_t
	modif = -1;

#endif	/* EXT_EEPROM */

#endif	/* DATA_FLASH_LINKED */

abyte_t
	eqp_flg;

/******************************************************************************
Private functions
******************************************************************************/

#ifdef DATA_FLASH_LINKED

/* data-flash management functions *******************************************/

/******************************************************************************
* Function Name: read_eqp
* Description  : data flash reading
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if errors
******************************************************************************/
static uint16_t read_eqp(void)
{
	uint16_t	par_a, eqp_a;
	df_val_t	eqp_val;

	FAL_Open();
	par_a = 0;
	for (eqp_a = 0; eqp_a < NUM_EQP; eqp_a++)
	{
		READ( (df_read_add + eqp_a) )
		if ( READNOK )
		{
			SET_EQP_ERROR
			FAL_Close();
			return (1);
		}
		eqp_val.longval = READVAL;

		if (eqp_val.wordval.lower < par_min[par_a])
		{
			FAL_Close();
			return (1);
		}
		if (eqp_val.wordval.lower > par_max[par_a])
		{
			FAL_Close();
			return (1);
		}
		par_vec[par_a++] = eqp_val.wordval.lower;

		if (eqp_val.wordval.higher < par_min[par_a])
		{
			FAL_Close();
			return (1);
		}
		if (eqp_val.wordval.higher > par_max[par_a])
		{
			FAL_Close();
			return (1);
		}
		par_vec[par_a++] = eqp_val.wordval.higher;
	}
	FAL_Close();
	return (0);

}	/* end of function read_eqp() */

/******************************************************************************
* Function Name: write_eqp
* Description  : data flash writing
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if errors
******************************************************************************/
static uint16_t write_eqp(void)
{
	uint16_t	par_a, eqp_a;
	df_val_t	eqp_val;

	FAL_Open();
	par_a = 0;
        
	for (eqp_a = 0; eqp_a < NUM_EQP; eqp_a++)
	{
		eqp_val.wordval.lower = par_vec[par_a++];
		eqp_val.wordval.higher = par_vec[par_a++];
		WRITE( (df_write_add + eqp_a), eqp_val.longval )
		WAITBUSY
                  
		if ( WRITENOK )
		{
			SET_EQP_ERROR
			FAL_Close();
			return (1);
		}
	}
        
	eqp_val.longval = CHKVAL;
	WRITE( (df_write_add + 15), eqp_val.longval )
	WAITBUSY
          
	if ( WRITENOK )
	{
		SET_EQP_ERROR
		FAL_Close();
		return (1);
	}
        
	FAL_Close();
	df_read_add = df_write_add;
	return (0);

}	/* end of function write_eqp() */

/******************************************************************************
* Function Name: erase_eqp
* Description  : data flash erasing
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if errors
******************************************************************************/
static uint16_t erase_eqp(void)
{
	uint16_t blk_a;

	df_read_add = 32768;
	df_write_add = 32768;

	FAL_Open();
        
	for (blk_a = 0; blk_a < FAL_POOL_SIZE; blk_a++)
	{
		ERASE( blk_a )
		WAITBUSY
                  
		if ( ERASENOK )
		{
			SET_EQP_ERROR
			FAL_Close();
			return (1);
		}
	}
        
	FAL_Close();
	df_write_add = 0;
	return (0);

}	/* end of function erase_eqp() */

/* data-flash management functions end ****************************************/
#else	/* DATA_FLASH_LINKED */

#ifdef EXT_EEPROM
/* management functions for external iic eeprom *******************************/

/******************************************************************************
* Function Name: eqp_hw_ini
* Description  : hardware initialization
* Arguments    : none
* Return Value : none
******************************************************************************/
static void eqp_hw_ini(void)
{

	PUT_SCL_OUT			// I/O settings
	CLR_SCL
	SET_SDA

}	/* end of function eqp_hw_ini() */

/******************************************************************************
* Function Name: send_start
* Description  : sends start sequence
* Arguments    : none
* Return Value : none
******************************************************************************/
static void send_start(void)
{

	CLR_SCL				// for safety
	SET_SDA				// for safety
	WAIT_SCL
	SET_SCL
	WAIT_SCL
	CLR_SDA
	WAIT_SCL
	CLR_SCL
	SET_SDA

}	/* end of function send_start() */

/******************************************************************************
* Function Name: send_stop
* Description  : sends stop sequence
* Arguments    : none
* Return Value : none
******************************************************************************/
static void send_stop(void)
{

	CLR_SCL				// for safety
	CLR_SDA
	WAIT_SCL
	SET_SCL
	WAIT_SCL
	SET_SDA
	WAIT_SCL
	CLR_SCL

}	/* end of function send_stop() */

/******************************************************************************
* Function Name: send_byte
* Description  : sends a byte
* Arguments    : byte to be sent
* Return Value : 0 if all OK, NOTZERO if errors
******************************************************************************/
static uint16_t send_byte(uint8_t byt)
{
	uint8_t i;

	CLR_SCL				// for safety
          
	for (i = 0; i < 8; i++, byt <<= 1)
	{
		if (byt & 0x80)
		{
			SET_SDA
		}
		else
		{
			CLR_SDA
		}
		WAIT_SCL
		SET_SCL
		WAIT_SCL
		CLR_SCL
	}
        
	SET_SDA
	WAIT_SCL
	SET_SCL
	WAIT_SCL
	
        if (SDA_IN_L)
	{
		CLR_SCL
		return (0);
	}
	else
	{
		CLR_SCL
		send_stop();
		return (1);
	}

}	/* end of function send_byte(...) */

/******************************************************************************
* Function Name: read_byte
* Description  : receives a byte
* Arguments    : ack to be asserted or not
* Return Value : read value
******************************************************************************/
static uint8_t read_byte(uint8_t ack)
{
	uint8_t i, msk, val=0;

	CLR_SCL				// for safety
	SET_SDA				// for safety
          
	for (i = 0, msk = 0x80; i < 8; i++, msk >>= 1)
	{
		WAIT_SCL
		SET_SCL
		WAIT_SCL
		if (SDA_IN_H)
			val |= msk;
		CLR_SCL
	}
        
	if (ack)
	{
		CLR_SDA
	}
	else
	{
		SET_SDA
	}
        
	WAIT_SCL
	SET_SCL
	WAIT_SCL
	CLR_SCL
	SET_SDA
	return (val);

}	/* end of function read_byte(...) */

/******************************************************************************
* Function Name: eqp_write_word
* Description  : writes a word data in eeprom
* Arguments    : eeprom address to write to, value to be written (both uint16_t)
* Return Value : 0 if all OK, NOTZERO if errors
******************************************************************************/
static uint16_t eqp_write_word(uint16_t add, uint16_t val)
{
	uint8_t msb, lsb;

	if (add > MAX_EQP)
	{
		return (1);
	}
	add <<= 1;					// word access
	send_start();
        
	if (0 != send_byte(CODE_ADD_W))
	{
		return (1);
	}
        
	msb = add >> 8;					// MSB of address
	lsb = add & 0x00FF;				// LSB of address
        
#ifdef	ADD2BYTES
	if (0 != send_byte(msb))
	{
		return (1);
	}
#endif
        
	if (0 != send_byte(lsb))
	{
		return (1);
	}
        
	msb = val >> 8;					// MSB of value
	lsb = val & 0x00FF;				// LSB of value
        
	if (0 != send_byte(msb))
	{
		return (1);
	}
        
	if (0 != send_byte(lsb))
	{
		return (1);
	}
        
	send_stop();
	return (0);

}	/* end of function eqp_write_word(...) */

/******************************************************************************
* Function Name: eqp_read_word
* Description  : reads a word from eeprom
* Arguments    : eeprom address to be read, address of the return value
* Return Value : 0 if all OK, NOTZERO if errors
******************************************************************************/
static uint16_t eqp_read_word(uint16_t add, uint16_t *val)
{
	uint8_t		msb, lsb;
	uint16_t	result;

	if (add > MAX_EQP)
	{
		return (1);
	}
        
	add <<= 1;					// word access
	send_start();
        
	if (0 != send_byte(CODE_ADD_W))
	{
		return (1);
	}
        
	msb = add >> 8;					// MSB of address
	lsb = add & 0x00FF;				// LSB of address
        
#ifdef	ADD2BYTES
	
        if (0 != send_byte(msb))
	{
		return (1);
	}
#endif
	if (0 != send_byte(lsb))
	{
		return (1);
	}
        
	send_start();
	
        if (0 != send_byte(CODE_ADD_R))
	{
		return (1);
	}
        
	msb = read_byte(1);			// MSB of value
	lsb = read_byte(0);			// LSB of value
	send_stop();
	result = msb;
	result <<= 8;
	result |= lsb;
	*val = result;
	return (0);

}	/* end of function eqp_read_word(...) */

/******************************************************************************
* Function Name: eqp_busy
* Description  : test busy
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if busy or errors
******************************************************************************/
static uint16_t eqp_busy(void)
{
	send_start();
        
	if (0 != send_byte(CODE_ADD_W))
	{
		return (1);
	}
        
	send_stop();
	return (0);

}	/* end of function eqp_busy() */

/******************************************************************************
* Function Name: wait_busy
* Description  : waits if eeprom is busy, for a max of max_time delays
* Arguments    : waiting time in millisec
* Return Value : 0 if all OK, NOTZERO if busy or errors
* Note         : to be used out of main loop, due to the high delay
******************************************************************************/
static uint16_t wait_busy(uint16_t max_time)
{
	if (EQP_ERROR)
	{
		return (1);
	}
        
	while (eqp_busy())
	{
		if (max_time != 0)
		{
			DELAY100US(10)		// 1ms
			--max_time;
		}
		else
		{
			SET_EQP_ERROR
			return (1);
		}
	}
	return (0);

}	/* end of function wait_busy(...) */

/******************************************************************************
* Function Name: ram_to_eqp
* Description  : copies all the parameters from ram to eeprom
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if errors
* Note         : to be used out of main loop, due to the high delay
******************************************************************************/
static uint16_t ram_to_eqp(void)
{
	static uint16_t i;

	/* wait if busy */
	if (wait_busy(MAX_WAIT))
	{
		SET_EQP_ERROR
		return (1);
	}

	/* read/write cycle */
	for (i = 0; i < NUM_EQP; i++) {
		if (eqp_read_word(i, &eqp_aus))
		{
			SET_EQP_ERROR
			return (1);
		}
		if (par_vec[i] != eqp_aus) {
			if (eqp_write_word(i, par_vec[i]))
			{
				SET_EQP_ERROR
				return (1);
			}
			if (wait_busy(MAX_WAIT))
			{
				SET_EQP_ERROR
				return (1);
			}
			if (eqp_read_word(i, &eqp_aus))
			{
				SET_EQP_ERROR
				return (1);
			}
			if (par_vec[i] != eqp_aus)
			{
				SET_EQP_ERROR
				return (1);
			}
		}
	}
	return (0);

}	/* end of function ram_to_eqp() */

/* end of management functions for iic_eeprom *********************************/
#endif 	/* EXT_EEPROM */

#endif	/* DATA_FLASH_LINKED */

/******************************************************************************
Exported functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
* Function Name: def_to_ram
* Description  : copies default parameters into ram
* Arguments    : none
* Return Value : none
******************************************************************************/
void def_to_ram(void)
{
	uint16_t i;

	for (i = 0; i < NUM_PAR; i++)
	{
		par_vec[i] = par_def[i];
	}

}	/* end of function def_to_ram() */

#ifdef DATA_FLASH_LINKED
/* management functions when data flash routines are linked */

/******************************************************************************
* Function Name: ini_eqp
* Description  : libray functions init
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if errors
* Note         : to be used out of main loop, due to the high delay
******************************************************************************/
uint16_t ini_eqp(void)
{
	uint16_t		read_a, flg_a, i;
	fal_status_t	df_status;

	def_to_ram();
	RES_EQP_ERROR
	RES_EQP_MODREQ
	RES_EQP_ERASING
	SET_EQP_UPDATED

	df_status = FAL_Init( &fal_descriptor_str );
        
	if (df_status != FAL_OK)
	{
		SET_EQP_ERROR
		return (1);
	}

	read_a = 0;
	flg_a = 31;
	BLANKCHK( flg_a )
	WAITBUSY
          
	while (NOTBLANK && (flg_a < 1023))
	{
		read_a += 32;
		flg_a += 32;
		BLANKCHK( flg_a )
		WAITBUSY
	}
        
	if (BLANK)
	{
		if (0 == read_a)
		{
			// all blank: for safety erase all blocks, then write default parameters
			if (erase_eqp())
			{
				SET_EQP_ERROR
				return (1);		// exit with alarm
			}
			df_write_add = 0;
			if (write_eqp())
			{
				SET_EQP_ERROR
				return (1);		// exit with alarm
			}
			df_read_add = 0;
			return (0);
		}
	}

	df_read_add = read_a;
        
	if (read_eqp())
	{
		SET_EQP_ERROR
		return (1);		// exit with alarm
	}

	/* default values reload management */
	if (RIP_DEF == par_vec[0])
	{
		def_to_ram();
		df_write_add = df_read_add + 32;
		if (df_write_add > (1024-32))
		{
			if (erase_eqp())
			{
				SET_EQP_ERROR
				return (1);		// exit with alarm
			}
			df_write_add = 0;
		}
		if (write_eqp())
		{
			SET_EQP_ERROR
			return (1);		// exit with alarm
		}
		df_read_add = df_write_add;
		return (0);
	}

	/* parameters control */
	for (i = 1; i < NUM_EQP; i++)
	{
		if ( (par_vec[i] < par_min[i]) || (par_vec[i] > par_max[i]) )
		{
			def_to_ram();
			df_write_add = df_read_add + 32;
			if (df_write_add > (1024-32))
			{
				if (erase_eqp())
				{
					SET_EQP_ERROR
					return (1);		// exit with alarm
				}
				df_write_add = 0;
			}
			if (write_eqp())
			{
				SET_EQP_ERROR
				return (1);		// exit with alarm
			}
			df_read_add = df_write_add;
			return (1);			// without setting EQP_ERROR
		}
	}

	/* current PIs tuning mode management */
	if (TUN_CPI == par_vec[0])
	{
		SET_CURPI_TUN
		par_vec[0] = 0;
		df_write_add = df_read_add + 32;
		if (df_write_add > (1024-32))
		{
			if (erase_eqp())
			{
				SET_EQP_ERROR
				return (1);		// exit with alarm
			}
			df_write_add = 0;
		}
		if (write_eqp())
		{
			SET_EQP_ERROR
			return (1);		// exit with alarm
		}
		df_read_add = df_write_add;
	}

	return (0);

}	/* end of function ini_eqp() */

/******************************************************************************
* Function Name: man_eqp
* Description  : parameters update management
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if errors
******************************************************************************/
uint16_t man_eqp(void)
{
	df_val_t	eqp_val;

	if (EQP_ERROR)
	{
		return (0);
	}
        
	if ( EQPBUSY )
	{
		return (0);
	}
        
	if (EQP_UPDATED)
	{
		if (! EQP_MODREQ)
		{
			return (0);
		}
		FAL_Open();
		RES_EQP_UPDATED
		RES_EQP_MODREQ
		eqp_add = 0;
		par_add = 0;
		blk_add = 0;
		df_write_add = df_read_add + 32;
		if (df_write_add > (1024-32))
		{
			SET_EQP_ERASING
			df_read_add = 32768;
			df_write_add = 32768;
			ERASE( blk_add )
			++blk_add;
			return (0);
		}
	}
        
	if (EQP_ERASING)
	{
		if ( ERASENOK )
		{
			FAL_Close();
			RES_EQP_ERASING
			SET_EQP_UPDATED
			SET_EQP_ERROR
			return (1);
		}
		if (blk_add < FAL_POOL_SIZE)
		{
			ERASE( blk_add )
			++blk_add;
			return (0);
		}
		RES_EQP_ERASING
		df_write_add = 0;
		// no return
	}
        
	if ( WRITENOK )
	{
		FAL_Close();
		SET_EQP_UPDATED
		SET_EQP_ERROR
		return (1);
	}
        
	if (eqp_add < NUM_EQP)
	{
		eqp_val.wordval.lower = par_vec[par_add++];
		eqp_val.wordval.higher = par_vec[par_add++];
		WRITE( (df_write_add + eqp_add), eqp_val.longval )
		++eqp_add;
		return (0);
	}
        
	df_read_add = df_write_add;
	FAL_Close();
	SET_EQP_UPDATED
	return (0);

}	/* end of function man_eqp() */

/******************************************************************************
* Function Name: req_eqp
* Description  : parameter update request
* Arguments    : new parameter value, parameter address
* Return Value : none
******************************************************************************/
void req_eqp(uint16_t data, uint16_t addr)
{

	if ((TRIG_ADD == addr) && (0 != data))
	{
		SET_CURPI_TRI
	}
	else if (data != par_vec[addr])
	{
		if (data >= par_min[addr])
		{
			if (data <= par_max[addr])
			{
				par_vec[addr] = data;
				SET_EQP_MODREQ
			}
		}
	}

}	/* end of function req_eqp(...) */

/* end of management functions when data flash routines are linked */
#else	/* DATA_FLASH_LINKED */

#ifdef EXT_EEPROM
/* management functions if external eeprom is used */

/******************************************************************************
* Function Name: ini_eqp
* Description  : libray functions init
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if errors
* Note         : to be used out of main loop, due to the high delay
******************************************************************************/
uint16_t ini_eqp(void)
{
	uint16_t	i;

	def_to_ram();
	eqp_hw_ini();
	RES_EQP_ERROR

	/* wait if busy */
	if (wait_busy(MAX_WAIT))
	{
		SET_EQP_ERROR
		return (1);
	}

	/* parameters reading */
	for (i = 0; i < NUM_EQP; i++)
	{
		if (eqp_read_word(i, &eqp_aus))
		{
			SET_EQP_ERROR
			return (1);
		}
		par_vec[i]=eqp_aus;
	}

	/* default values restoration if requested */
	if ((RIP_DEF == par_vec[0]) || (0xFFFF == par_vec[0]))
	{
		def_to_ram();
		if (ram_to_eqp())
		{
			SET_EQP_ERROR
			return (1);
		}
		return (0);
	}

	/* parameters control */
	for (i = 1; i < NUM_EQP; i++)
	{
		if ( (par_vec[i] < par_min[i]) || (par_vec[i] > par_max[i]) )
		{
			def_to_ram();
			if (ram_to_eqp())
			{
				SET_EQP_ERROR
			}
			return (1);		// without setting EQP_ERROR
		}
	}

	/* current PIs tuning mode management */
	if (TUN_CPI == par_vec[0]) {
		SET_CURPI_TUN
		par_vec[0]=0;
                
		if (eqp_write_word(0, par_vec[0]))
		{
			SET_EQP_ERROR
			return (1);
		}
                
		if (wait_busy(MAX_WAIT))
		{
			SET_EQP_ERROR
			return (1);
		}
                
		if (eqp_read_word(0, &eqp_aus))
		{
			SET_EQP_ERROR
			return (1);
		}
                
		if (eqp_aus != 0)
		{
			SET_EQP_ERROR
			return (1);
		}
	}

	return (0);

}	/* end of function ini_eqp() */

/******************************************************************************
* Function Name: man_eqp
* Description  : parameters update management
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if errors
******************************************************************************/
uint16_t man_eqp(void)
{
	uint16_t data, addr;

	if (EQP_ERROR)
	{
		return (0);
	}
        
	if (modif < 0)
	{
		if (sla_ind != mas_ind)
		{
			data = eqp_fifo[sla_ind];
			addr = eqp_add[sla_ind];
			eqp_fifo[sla_ind] = 0;
			eqp_add[sla_ind++] = 0;
			if (sla_ind >= NUM_PAR)
			{
				sla_ind = 0;
			}
			if ((data >= par_min[addr]) && (data <= par_max[addr]))
			{
				par_vec[addr] = data;
				modif = addr;
			}
		}
		return (0);
	}
        
	if (eqp_bsy)
	{
		if (eqp_busy())
		{
			if (0 == (--eqp_bsy))
			{
				modif = -1;
				SET_EQP_ERROR
				return (1);
			}
			return (0);
		}
                
		eqp_bsy = 0;
                
		if (eqp_read_word(modif, &eqp_aus))
		{
			modif = -1;
			SET_EQP_ERROR
			return (1);
		}
                
		if (par_vec[modif] != eqp_aus)
		{
			modif = -1;
			SET_EQP_ERROR
			return (1);
		}
                
		modif = -1;
		return (0);
	}
        
	if (modif < NUM_EQP)
	{
		if (eqp_read_word(modif, &eqp_aus))
		{
			modif = -1;
			SET_EQP_ERROR
			return (1);
		}
		else if (par_vec[modif] != eqp_aus)
		{
			if (eqp_write_word(modif, par_vec[modif]))
			{
				modif = -1;
				SET_EQP_ERROR
				return (1);
			}
                        
			eqp_bsy = NL_W_EQP;
			return (0);
		}
	}
        
	modif = -1;
	return (0);

}	/* end of function man_eqp() */

/******************************************************************************
* Function Name: req_eqp
* Description  : parameter update request
* Arguments    : new parameter value, parameter address
* Return Value : none
******************************************************************************/
void req_eqp(uint16_t data, uint16_t addr)
{

	if ((TRIG_ADD == addr) && (0 != data))
	{
		SET_CURPI_TRI
	}
	else if (data != par_vec[addr])
	{
		if (data >= par_min[addr])
		{
			if (data <= par_max[addr])
			{
				eqp_fifo[mas_ind] = data;
				eqp_add[mas_ind++] = addr;
				if (mas_ind >= NUM_PAR)
				{
					mas_ind = 0;
				}
			}
		}
	}

}	/* end of function req_eqp(...) */

/* end of management functions if external eeprom is used */
#else	/* EXT_EEPROM */

/* management functions if no eeprom is used */

/******************************************************************************
* Function Name: ini_eqp
* Description  : init parameters table
* Arguments    : none
* Return Value : 0 (means all OK)
******************************************************************************/
uint16_t ini_eqp(void)
{

	def_to_ram();
	return (0);

}	/* end of function ini_eqp() */

/******************************************************************************
* Function Name: man_eqp
* Description  : does nothin
* Arguments    : none
* Return Value : 0
******************************************************************************/
uint16_t man_eqp(void)
{

	return (0);

}	/* end of function man_eqp() */

/******************************************************************************
* Function Name: req_eqp
* Description  : parameter update
* Arguments    : new parameter value, parameter address
* Return Value : none
******************************************************************************/
void req_eqp(uint16_t data, uint16_t addr)
{
	uint16_t i;

	if (0 == addr)
	{
		if (RIP_DEF == data)
		{
			for (i = 0; i < NUM_PAR; i++)
			{
				par_vec[i] = par_def[i];
			}
			return;
		}
		else if (TUN_CPI == data)
		{
			par_vec[0] = TUN_CPI;
			SET_CURPI_TUN
			return;
		}
		else
		{
			RES_CURPI_TUN
			if (data >= par_min[0])
			{
				if (data <= par_max[0])
				{
					par_vec[0] = data;
				}
			}
			return;
		}
	}
	else if ((TRIG_ADD == addr) && (0 != data))
	{
		SET_CURPI_TRI
	}
	else if (data != par_vec[addr])
	{
		if (data >= par_min[addr])
		{
			if (data <= par_max[addr])
			{
				par_vec[addr] = data;
			}
		}
	}

}	/* end of function req_eqp(...) */

/* end management functions if no eeprom is used */
#endif	/* EXT_EEPROM */
#endif	/* DATA_FLASH_LINKED */
