/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
*******************************************************************************/
/* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.   */
/*******************************************************************************
* File Name    : par_tab.h
* Version      : 2.0
* Description  : global includes and defines
* Device(s)    : RL78/G14
* Tool-Chain   : RL78 IAR Embedded Workbench V1.20.x
* OS           : None
* H/W Platform : YRMCKITRL78G14 Rev 2.0
* Description  : RL78 IEC Self Test Library Header File
* Operation    :
* Limitations  : None
********************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0    Production release plus rebuild for new IAR EW release
*                              modified file organisation to support object creation
*******************************************************************************/

#ifndef _PAR_TAB_H
#define _PAR_TAB_H

/******************************************************************************
Includes
******************************************************************************/
#include "typedefine.h"


/******************************************************************************
Macro definitions
******************************************************************************/

#define NUM_PAR 	(64)		// parameters number
#define NUM_USED 	(20)		// used parameters number
#define RIP_DEF 	(33)		// default parameters setting password
#define TUN_CPI 	(22)		// current PI tuning password
#define TRIG_ADD	(17)		// trigger parameter address

#define SEL_OP		par_vec[0]	// default parameters setting
#define RPM_MIN		par_vec[1]	// minimum speed (rpm)
#define RPM_MAX		par_vec[2]	// maximum speed (rpm)
#define R_ACC		par_vec[3]	// acceleration	(rpm/s)
#define R_DEC		par_vec[4]	// deceleration	(rpm/s)
#define C_POLI		par_vec[5]	// polar couples 
#define I_START		par_vec[6]	// startup current (Ampere/AMP_DIV)
#define I_MAX		par_vec[7]	// maximum current (Ampere/AMP_DIV)
#define R_STA		par_vec[8]	// stator resistance (Ohm/OHM_DIV)
#define L_SYN		par_vec[9]	// synchronous inductance (Henry/HEN_DIV)
#define STP_TIM		par_vec[10]	// startup time	(ms)
#define KP_CUR		par_vec[11]	// current loop KP
#define KI_CUR		par_vec[12]	// current loop KI
#define KP_VEL		par_vec[13]	// speed loop KP
#define	KI_VEL		par_vec[14]	// speed loop KI
#define	PAR_15		par_vec[15]	//
#define	PAR_16		par_vec[16]	//
#define PAR_17		par_vec[17]	//
#define PAR_18		par_vec[18]	//
#define PAR_19		par_vec[19]	//
#define	PAR_20		par_vec[20]	//
#define PAR_21		par_vec[21]	//
#define PAR_22		par_vec[22]	//
#define PAR_23		par_vec[23]	//
#define PAR_24		par_vec[24]	//
#define PAR_25		par_vec[25]	//
#define PAR_26		par_vec[26]	//
#define PAR_27		par_vec[27]	//
#define PAR_28		par_vec[28]	//
#define PAR_29		par_vec[29]	//
#define PAR_30		par_vec[30]	//
#define PAR_31		par_vec[31]	//
#define PAR_32		par_vec[32]	//
#define PAR_33		par_vec[33]	//
#define PAR_34		par_vec[34]	//
#define PAR_35		par_vec[35]	//
#define PAR_36		par_vec[36]	//
#define PAR_37		par_vec[37]	//
#define PAR_38		par_vec[38]	//
#define PAR_39		par_vec[39]	//
#define PAR_40		par_vec[40]	//
#define PAR_41		par_vec[41]	//
#define PAR_42		par_vec[42]	//
#define PAR_43		par_vec[43]	//
#define PAR_44		par_vec[44]	//
#define PAR_45		par_vec[45]	//
#define PAR_46		par_vec[46]	//
#define PAR_47		par_vec[47]	//
#define PAR_48		par_vec[48]	//
#define PAR_49		par_vec[49]	//
#define PAR_50		par_vec[50]	//
#define PAR_51		par_vec[51]	//
#define PAR_52		par_vec[52]	//
#define PAR_53		par_vec[53]	//
#define PAR_54		par_vec[54]	//
#define PAR_55		par_vec[55]	//
#define PAR_56		par_vec[56]	//
#define PAR_57		par_vec[57]	//
#define PAR_58		par_vec[58]	//
#define PAR_59		par_vec[59]	//
#define PAR_60		par_vec[60]	//
#define PAR_61		par_vec[61]	//
#define PAR_62		par_vec[62]	//
#define PAR_63		par_vec[63]	//

/******************************************************************************
External Variable references
******************************************************************************/

extern const uint16_t
	par_min[NUM_PAR],
	par_max[NUM_PAR],
	par_def[NUM_PAR];

extern uint16_t
	par_vec[NUM_PAR];


/******************************************************************************
Functions Prototypes
******************************************************************************/

/******************************************************************************
* Function Name: ini_eqp
* Description  : libray functions init
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if errors
* Note         : to be used out of main loop, due to the high delay
******************************************************************************/
uint16_t ini_eqp(void);


/******************************************************************************
* Function Name: man_eqp
* Description  : parameters update management
* Arguments    : none
* Return Value : 0 if all OK, NOTZERO if errors
******************************************************************************/
uint16_t man_eqp(void);


/******************************************************************************
* Function Name: req_eqp
* Description  : parameter update request
* Arguments    : new parameter value, parameter address
* Return Value : none
******************************************************************************/
void req_eqp(uint16_t data, uint16_t add);

#endif /* _PAR_TAB_H */
