
/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
*******************************************************************************/
/* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.   */
/*******************************************************************************
* File Name : stl_support.c
* Version : 2.0
* Device(s) : RL78/G14
* Tool-Chain : RL78 IAR Embedded Workbench V1.20.x
* OS : None
* H/W Platform : YRMCKITRL78G14 V2.0
* Description : RL78 IEC Self Test Library Main test harness support functions
*               Source Build
* Operation : This module provides support function for the self-test modules
* Limitations : None
********************************************************************************
* History :
* 30.05.2011 V1.0 First Release (Prototype)
* 01.06.2012 V1.1 Updated prototype release to include motor control library
                  plus added build option for RL78G14 64 pin device
* 15.06.2012 V2.0 Production release plus rebuild for new IAR EW release
*                 modified file organisation to support object creation
*******************************************************************************/


/*******************************************************************************
Includes <System Includes> , Project Includes
*******************************************************************************/
#ifdef RL78G14_64

#include <ior5f104le.h>
#include <ior5f104le_ext.h>

#else

#include <ior5f104pj.h>
#include <ior5f104pj_ext.h>

#endif

#include <intrinsics.h>
#include <stdbool.h>
#include <stdint.h>

/******************************************************************************
 Exported global variables and functions (to be accessed by other files)
******************************************************************************/ 
uint16_t J;
uint8_t  g_referenceCalc_Done;

/******************************************************************************
 modul global variables and functions 
******************************************************************************/ 
static uint16_t crc_table[256];
static uint16_t poly = 0x1021U;
static void GenerateCRCTable(void);
static uint8_t reverse8(uint8_t value);
static uint16_t reverse16(uint16_t value);
       
/******************************************************************************
Imported global variables and functions (to be accessed by other files)
******************************************************************************/
extern bool g_resultBool;

/*******************************************************************************
Function Prototypes
*******************************************************************************/
void set_test_error (bool resultBool);
uint16_t reference_crc_calculation(uint16_t crc_input, uint32_t length, uint8_t __far *p);

/******************************************************************************
* Outline : indicate_test_result (bool g_resultBool)
******************************************************************************/      
void set_test_error(bool resultBool)
{
	if(resultBool)
	{
		g_resultBool = true;
	}
	else
	{
		g_resultBool = false;
	}
}

/******************************************************************************
* Outline : GenerateCRCTable();
* Include : intrinics.h, ior5f100le.h, ior5f100le_ext.h, stdbool.h, stdint.h
* Declaration : void GenerateCRCTable(void)
* Function Name: GenerateCRCTable
* Description : Generates a reference CRC table for use with the Hardware 
                peripheral CRC to the defined polynomial (poly)
* Argument : none
* Return Value : none 
* Calling Functions : stl_main_example_support_functions.c
******************************************************************************/      
static void GenerateCRCTable(void)
{
      /* Temporary Variables */
      uint16_t temp, a;
      
      /* Create table to Polynomial supplied in this file */
      for (uint16_t i = 0U; i < 256U; ++i)
      {
          temp = 0U;
          a = (unsigned short)(i << 8U);
          for (int16_t j = 0; j < 8; ++j)
          {
              if (((temp ^ a) & 0x8000) != 0U)
              {
                  temp = (uint16_t)((temp << 1U) ^ poly);
              }
              else
              {
                  temp <<= 1U;
              }
              a <<= 1;
          }
          crc_table[i] = temp;      /* Add value to reference CRC table */
      }
}

/******************************************************************************
* Outline : reverse16(uint16_t value);
* Include : intrinics.h, ior5f100le.h, ior5f100le_ext.h, stdbool.h, stdint.h
* Declaration : uint16_t reverse16(uint16_t value)
* Function Name: reverse16
* Description : Bit reverse the 16 bit input value 
* Argument : uint16_t value - 16bit input value
* Return Value : uint16_t valueout - Bit swapped 16bit value
* Calling Functions : stl_main_example_support_functions.c
******************************************************************************/      
static uint16_t reverse16(uint16_t value)
{
    /* Temporary Variables */
    uint16_t i, j = 1U, valueout = 0U;

   /* Function to bit swap the 16bit input value */
   for (i = (uint16_t)1 << 15; i != 0U; i >>= 1U)
   {
       if ((value & i) != 0U)
       {
           valueout |= j;
       }
           j <<= 1;
   }
    
   return (valueout);               /* Return bit swapped value */
}


/******************************************************************************
* Outline : reverse8(uint8_t value);
* Include : intrinics.h, ior5f100le.h, ior5f100le_ext.h, stdbool.h, stdint.h
* Declaration : uint8_t reverse8(uint8_t value)
* Function Name: reverse16
* Description : Bit reverse the 8 bit input value 
* Argument : uint8_t value - 8 bit input value
* Return Value : uint8_t valueout - Bit swapped 8 bit value
* Calling Functions : stl_main_example_support_functions.c
******************************************************************************/      
static uint8_t reverse8(uint8_t value)
{
   /* Temporary Variables */
   uint8_t i, j = 1U, valueout = 0U;
    

   /* Function to bit swap the 16bit input value */
   for (i = (uint16_t)1 << 7U; i != 0U; i >>= 1U)
   {
        if ((value & i) != 0U)
        {
            valueout |= j;
        }
        j <<= 1;
   }

   return (valueout);               /* Return bit swapped value */
}

/******************************************************************************
* Outline : reference_crc_calculation
* Include : intrinics.h, ior5f100le.h, ior5f100le_ext.h, stdbool.h, stdint.h
* Declaration : uint16_t reference_crc_calculation(uint16_t crc_input, uint32_t length, uint8_t __far *p)
* Function Name: reference_crc_calculation
* Description : Calculate reference CRC value of the specified memory area
                The CRC generation is compatible with the RL78 CRC peripheral 
                (Peripheral mode)
* Argument : uint16_t crc_input - Starting CRC Value
             uint32_t length - Length of CRC calculation range 
             uint8_t __far *p - Start address pointer
* Return Value : uint16_t crc_calc - Calculated reference CRC result value
* Calling Functions : main.c
******************************************************************************/      
uint16_t reference_crc_calculation(uint16_t crc_input, uint32_t length, uint8_t __far *p)
{
   /* Temporary Variables */
   uint8_t byte = 0U;
   uint32_t i;
   uint16_t crc_calc;
      
   /* Generate reference table */
   GenerateCRCTable();


   /* Buffer initial CRC start value*/
   crc_calc = crc_input;
      

    /* Calculate CRC */
    for ( i = 0U; i < length; ++i)
    {
         byte = *p++;
         byte = reverse8(byte);
         crc_calc = crc_table[(crc_calc >> 8) ^ byte] ^ (crc_calc << 8);
    }
        
    /* Bit Swap result */
    crc_calc = reverse16(crc_calc);

    g_referenceCalc_Done = 1U;      /* Set reference CRC calculation done flag */

    return (crc_calc);              /* Return calculated reference CRC value */
}

