/******************************************************************************
* DISCLAIMER

* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized.

* This software is owned by Renesas Electronics Corporation and is protected under 
* all applicable laws, including copyright laws.

* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES 
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, 
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
* PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY 
* DISCLAIMED.

* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES 
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS 
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

* Renesas reserves the right, without notice, to make changes to this 
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and 
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer

******************************************************************************/

/******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : userif.h
* Version      : 2.0
* Device(s)    : RL78G14
* Tool-Chain   : RL78 IAR Embedded Workbench V1.20.x
* OS           : none
* H/W Platform : YRMCKITRL78G14 V2.0
* Description  : header for userif.c
* Operation    :
* Limitations  :
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0    Production release plus rebuild for new IAR EW release
*                              modified project organisation to support object creation
*****************************************************************************/

#ifndef _USERIF_H
#define _USERIF_H


/******************************************************************************
Macro definitions
******************************************************************************/
#define N_RAM_READ	20
#define N_RAM_WRITE	8


/******************************************************************************
Typedef definitions
******************************************************************************/
typedef union
{
	unsigned short		ram_tab[N_RAM_READ];
	struct
	{
		signed short	rif;		//  0 internal speed reference
		signed short	rpm;		//  1 measured/estimated speed
		signed short	fre;		//  2 applied frequency
		signed short	id;		//  3 direct current
		signed short	iq;		//  4 quadrature current
		signed short 	vd;		//  5 direct voltage
		signed short 	vq;		//  6 quadrature voltage
		signed short	vb;		//  7 bus voltage
		signed short	all;		//  8 alarm
		signed short	flg;		//  9 status flags
		signed short	du1;		// 10 algo type index (GUI)
		signed short	du2;		// 11
		signed short	du3;		// 12
		signed short	du4;		// 13
		signed short	du5;		// 14
		signed short	du6;		// 15
		signed short	du7;		// 16
		signed short	du8;		// 17
		signed short	du9;		// 18
		signed short	du10;		// 19
	}					var;
}	UIF_R_t;

typedef union
{
	unsigned short	ram_tab[N_RAM_WRITE];
        
	struct
	{
		signed short	cco;		//  0
		signed short	flg;		//  1 flags
		signed short	rif;		//  2 speed reference
		signed short	du1;		//  3
		signed short	du2;		//  4
		signed short	du3;		//  5
		signed short	du4;		//  6
		signed short	du5;		//  7
	}					var;
}	UIF_W_t;


/******************************************************************************
Variable Externs
******************************************************************************/
extern UIF_R_t	  UIF_R;
extern UIF_W_t	  UIF_W;

/******************************************************************************
Functions Prototypes
******************************************************************************/
/******************************************************************************
* Function Name: MC_ReadButtons
* Description  : Buttons reading and debounce
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_ReadButtons(void);

/******************************************************************************
* Function Name: ini_232_pc
* Description  : serial communication init
* Arguments    : none
* Return Value : none
******************************************************************************/
void ini_232_pc(void);

/******************************************************************************
* Function Name: poll_232_pc
* Description  : serial communication polling
* Arguments    : none
* Return Value : none
* Note         : to be called in the control interrupt
******************************************************************************/
void poll_232_pc(void);

/******************************************************************************
* Function Name: ges_232_pc
* Description  : serial communication management
* Arguments    : none
* Return Value : none
* Note         : to be called in the main loop
******************************************************************************/
void ges_232_pc(void);

/******************************************************************************
* Function Name: ges_led
* Description  : led blinking management
* Arguments    : none
* Return Value : none
* Note         : to be called in the main loop
******************************************************************************/
void ges_led(void);



#endif /* _USERIF_H */
