;*******************************************************************************
;* DISCLAIMER
;* This software is supplied by Renesas Electronics Corporation and is only
;* intended for use with Renesas products. No other uses are authorized.
;* This software is owned by Renesas Electronics Corporation and is protected
;* under all applicable laws, including copyright laws.
;* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
;* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
;* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
;* PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY
;* DISCLAIMED.
;* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
;* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
;* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
;* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
;* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
;* Renesas reserves the right, without notice, to make changes to this
;* software and to discontinue the availability of this software.
;* By using this software, you agree to the additional terms and
;* conditions found by accessing the following link:
;* http://www.renesas.com/disclaimer
;*******************************************************************************
;* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.
;*******************************************************************************
;* File Name  : stl_RL78_march_c_initial.asm
;* Version    : 2.0
;* Device(s)  : RL78 (RL78/G14)
;* Tool-Chain : RL78 IAR Embedded Workbench V1.10.1
;* OS         : None
;* H/W Platform : YRMCKITRL78G14
;* Description : RL78 IEC Self Test Library March C test initial assembler module
;*               This modules is executed before the system is initialised
;*               so this test function is executed from the cstartup.s87 module
;*
;*      Register "AX" contains the start address of the RAM area to be tested
;*      Register "BC" contains the number of bytes(length) of the RAM area
;*      Registers AX, BC, DE, H of Bank1 are used to buffer
;       start address, stop, range, step indicator values of march_c test
;*
;*     Register "A" returns 0x01 if a memory fault was detected, otherwise
;*     0x00 is returned
;*
;* Limitations : The march_c_initial is a destructive RAM test
;*               To execute the test on all RAM location it should be run before
;*               the application system is initialised
;*               Interrupts should be disabled
;*******************************************************************************
;* History :
;* 30.05.2011 V1.0 First Release
;* 07.03.2012 V2.0 MarchC initial test module can possibly be out of range
;*                 Return branches modified to "far" to avoid test module 
;*                 potentially being out of range during link process
;*                 (Link order is aplhabetical)
;*******************************************************************************


        NAME    stl_RL78_march_c_initial             ; Label this program
        PUBLIC  stl_RL78_march_c_initial

        EXTERN  stl_RL78_march_c_initial_finish

        RSEG  CODE:CODE


;******************************************************************************
;* Outline : stl_RL78_march_c_initial Initial RL78 IEC March_c test module
;* Include : none
;* Declaration : stl_RL78_march_c_initial:
;* Function Name: stl_RL78_march_c_initial
;* Description : Initial IEC March_c test code module
;* Argument: Register "AX" contains the start address of the RAM area to be tested
;*           Register "BC" contains the number of bytes(length) of the RAM area
;*           Registers AX, BC, DE, H of Bank1 are used to buffer start, stop, length,
;*                     step indicator values of march_x test
;* Return Value : Register A - Result status (Pass = 0x00, Fail = 0x01)
;* Calling Functions : cstartup.s87
;******************************************************************************
; start of initial march_c module
stl_RL78_march_c_initial:
;* store input parameter
        MOVW  0xFEF2, AX        ; store start address of march c test into 0xFEF2 ( BC [Bank1] )
        MOVW  AX, BC            ; test length into AX
        MOVW  0xFEF0, AX        ; store test length into 0xFEF0 ( AX [Bank1] )

;* test if length is zero
        SUBW  AX, #0x0001       ; decrement test_length_value
        BNC   length_not_zero   ; if test_length_value was zero, c flag is set
        CLRB  A                 ; return value true - no error
        br    F:stl_RL78_march_c_initial_finish

;* start march -c test
length_not_zero:
        MOVW  AX, 0xFEF2        ; store start address of march c test into AX
        ADDW  AX, 0xFEF0        ; calculate stop address
        MOVW  0xFEF4, AX        ; store march-c end address into DE [Bank1]

;* clear error indicator ( reg D )
        CLRB  A                 ; clear register A
        MOV   D, A              ; clear D, which functions as error indicator

;* step 1 - <>(w0) - clear specified memory area
;* init step 1
        MOVW  HL, 0xFEF2        ; march-c start address from BC [Bank1] into HL register
        MOVW  AX, 0xFEF4        ; march-c end address from AX [Bank1] into AX register
;* test step 1
loop_step_1:
        MOV   [HL+0], #0x00     ; clear by HL addressed memory cell
        INCW  HL                ; increment RAM address register
        CMPW  AX, HL            ; check if all specified memory is cleared
        BZ    march_c_step_2    ; check if all RAM is cleared ( check length counter )
        BR    loop_step_1       ; next cycle

;* step 2 - >(r0,w1) - read bitwise if zero is in RAM, then write 1 into it, from lower address
;* init step 2
march_c_step_2:
        MOVW  AX, 0xFEF4        ; march-c end address from AX [Bank1] into AX register
        MOVW  HL, 0xFEF2        ; march-c start address from BC [Bank1] into HL register
        MOV   0xFEF7, #0x02     ; indicator to select return branch from r0_w1 into correct loop_step (2 or 4)

;* test step 2
loop_step_2:
        BR    r0_w1             ; branch to by [HL] addressed byte test and do r0 > w1
ret_2_4:                        ; return from r0 > w1 byte test
        CMP   0xFEF7, #0x04     ; check if r0 > w1 was branched to from loop_step_2 or loop_step_4
        BZ    ret_step_4        ; if it was branched to from loop_step_4 ( E=4 ) go there
ret_step_2:
        MOVW  AX, 0xFEF4        ; march-c end-address from AX [Bank1] into AX register
        INCW  HL                ; increment RAM address register
        CMPW  AX, HL            ; check if all specified memory is written one to
        BZ    march_c_step_3    ; check if all memory has been addressed
        BR    loop_step_2       ; not finished, next cycle

;* step 3 - >(r1,w0) - read bitwise if one is in RAM, then write zero into it, from lower address
;* init step 3
march_c_step_3:
        MOVW  AX, 0xFEF4        ; march-c end address from AX [Bank1] into AX register
        MOVW  HL, 0xFEF2        ; march-c start address from BC [Bank1] into HL register
        MOV   0xFEF7, #0x03     ; indicator to select return branch from r1_w0 into correct loop_step (3 or 5)

;* test step 3
loop_step_3:
        BR    r1_w0             ; branch to by [HL] addressed byte test and do r1 > w0
ret_3_5:
        CMP   0xFEF7, #0x05     ; check if r1 > w0 was branched to from loop_step_3 or loop_step_5
        BZ    ret_step_5        ; if it was branched to from loop_step_5 ( E=5 ) go there to continue
ret_step_3:
        MOVW  AX, 0xFEF4        ; march-c end address from AX [Bank1] into AX register
        INCW  HL                ; increment RAM address register
        CMPW  AX, HL            ; check if all specified memory is written zero to
        BZ    march_c_step_4    ; check if all RAM is cleared ( check length counter )
        BR    loop_step_3       ; not finished, next cycle

;* step 4 - <(r0,w1) - read bitwise if zero is in RAM, then write one into it, from upper address
;* init step 4
march_c_step_4:
        MOVW  AX, 0xFEF2        ; march-c start address from BC [Bank1] into AX register
        MOVW  HL, 0xFEF4        ; march-c end address from AX [Bank1] into HL register
        MOV   0xFEF7, #0x04     ; indicator to select return branch from r0_w1 into correct loop_step (2 or 4)

;* test step 4
loop_step_4:
        DECW  HL
        BR    r0_w1             ; branch to by [HL] addressed byte test and do r0 > w1
ret_step_4:
        MOVW  AX, 0xFEF2        ; march-c start address from BC [Bank1] into HL register
        CMPW  AX, HL            ; check if all specified memory is written one to
        BZ    march_c_step_5    ; check if all RAM is cleared ( check length counter )
        BR    loop_step_4       ; not finished, next cycle

;* step 5 - <(r1,w0) - read bitwise if one is in RAM, then write zero into it from upper address
;* init step 5
march_c_step_5:
        MOVW  AX, 0xFEF2        ; march-c start address from BC [Bank1] into AX register
        MOVW  HL, 0xFEF4        ; march-c end address from AX [Bank1] into HL register
        MOV   0xFEF7, #0x05     ; indicator to select return branch from r1_w0 into correct loop_step (3 or 5)

;* test step 5
loop_step_5:
        DECW  HL
        BR    r1_w0             ; branch to by [HL] addressed byte test and do r1 > w0
ret_step_5:
        MOVW  AX, 0xFEF2
        CMPW  AX, HL
        BZ    march_c_step_6    ; check if all RAM is cleared ( check length counter )
        BR    loop_step_5       ; next cycle

;* step 6 - <>(r0) - check if specified memory area is all zero
;* init step 6
march_c_step_6:
        MOVW  AX, 0xFEF4        ; march-c end address from AX [Bank1] into AX register
        MOVW  HL, 0xFEF2        ; march-c start address from BC [Bank1] into HL register

;* test step 6
loop_step_6:
        MOV   A, [HL]           ; load RAM cell addressed by HL
        CMP0  A                 ; check if RAM content is zero
        BZ    is_zero
        MOV   D, #0x01          ; D functions as error indicator
is_zero:
        MOVW  AX, 0xFEF4        ; march-c end address from AX [Bank1] into AX register
        INCW  HL                ; increment RAM address register
        CMPW  AX, HL
        BZ    out_6             ; check if all RAM is cleared ( check length counter )
        BR    loop_step_6       ; next cycle

;* move test result into register A and return it to calling function
out_6:
        MOV   A, D              ; return test result
        br    F:stl_RL78_march_c_initial_finish
                                ; test finished; test result in A, 01=error, 00=no error; return to c_startup


;* sub module to read by HL addressed ram, check bitwise if content is 0 and set bitwise to 1
r0_w1:
        MOV   A, [HL]           ; move byte to be tested into register A
        CMP   A, #00000000b     ; compare Register A with reference
        BZ    S_HL_0            ; if register content A equal reference value, no error - continue with next bit
        MOV   D, #0x01          ; if error occurred Register D carries error information 0x01
S_HL_0: set1  [HL].0
        MOV   A, [HL]
        CMP   A, #00000001b
        BZ    S_HL_1
        MOV   D, #0x01
S_HL_1: set1  [HL].1
        MOV   A, [HL]
        CMP   A, #00000011b
        BZ    S_HL_2
        MOV   D, #0x01
S_HL_2: set1  [HL].2
        MOV   A, [HL]
        CMP   A, #00000111b
        BZ    S_HL_3
        MOV   D, #0x01
S_HL_3: set1  [HL].3
        MOV   A, [HL]
        CMP   A, #00001111b
        BZ    S_HL_4
        MOV   D, #0x01
S_HL_4: set1  [HL].4
        MOV   A, [HL]
        CMP   A, #00011111b
        BZ    S_HL_5
        MOV   D, #0x01
S_HL_5: set1  [HL].5
        MOV   A, [HL]
        CMP   A, #00111111b
        BZ    S_HL_6
        MOV   D, #0x01
S_HL_6: set1  [HL].6
        MOV   A, [HL]
        CMP   A, #01111111b
        BZ    S_HL_7
        MOV   D, #0x01
S_HL_7: set1  [HL].7
        MOV   A, [HL]
        CMP   A, #11111111b
        BZ    S_HL_out
        MOV   D, #0x01
S_HL_out:
        BR    ret_2_4

;* sub module to read by HL addressed ram, check bitwise if content is 1 and set bitwise to 0
r1_w0:
        MOV   A, [HL]           ; move byte to be tested into register A
        CMP   A, #11111111b     ; compare Register A with reference
        BZ    C_HL_0            ; if register content A equal reference value, no error - continue with next bit
        MOV   D, #0x01          ; if error occurred Register D carries error information 0x01
C_HL_0: clr1  [HL].0
        MOV   A, [HL]
        CMP   A, #11111110b
        BZ    C_HL_1
        MOV   D, #0x01
C_HL_1: clr1  [HL].1
        MOV   A, [HL]
        CMP   A, #11111100b
        BZ    C_HL_2
        MOV   D, #0x01
C_HL_2: clr1  [HL].2
        MOV   A, [HL]
        CMP   A, #11111000b
        BZ    C_HL_3
        MOV   D, #0x01
C_HL_3: clr1  [HL].3
        MOV   A, [HL]
        CMP   A, #11110000b
        BZ    C_HL_4
        MOV   D, #0x01
C_HL_4: clr1  [HL].4
        MOV   A, [HL]
        CMP   A, #11100000b
        BZ    C_HL_5
        MOV   D, #0x01
C_HL_5: clr1  [HL].5
        MOV   A, [HL]
        CMP   A, #11000000b
        BZ    C_HL_6
        MOV   D, #0x01
C_HL_6: clr1  [HL].6
        MOV   A, [HL]
        CMP   A, #10000000b
        BZ    C_HL_7
        MOV   D, #0x01
C_HL_7: clr1  [HL].7
        MOV   A, [HL]
        CMP   A, #00000000b
        BZ    C_HL_out
        MOV   D, #0x01
C_HL_out:
        BR    ret_3_5

        END

